import boto3
from typing import List, Dict, Any
from datetime import datetime, timedelta
import json

class LBAuditService:
    def __init__(self):
        self.service_name = "LB"
        self.cost_checks = ['find_lbs_with_no_healthy_targets', 'find_underutilized_lbs', 'find_classic_lbs']
        self.security_checks = ['find_http_without_https_redirect', 'find_deprecated_tls_versions', 'find_lbs_without_access_logs']

    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all cost-related Load Balancer audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results

    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all security-related Load Balancer audits"""
        results = []
        for check in self.security_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results

    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all Load Balancer audits"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results

    def find_lbs_with_no_healthy_targets(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find load balancers with no healthy targets"""
        elbv2 = session.client('elbv2', region_name=region)
        results = []
        
        try:
            # Get all ALBs and NLBs
            lbs_response = elbv2.describe_load_balancers()
            
            for lb in lbs_response['LoadBalancers']:
                lb_arn = lb['LoadBalancerArn']
                
                # Get target groups for this LB
                tgs_response = elbv2.describe_target_groups(LoadBalancerArn=lb_arn)
                
                has_healthy_targets = False
                for tg in tgs_response['TargetGroups']:
                    # Check target health
                    health_response = elbv2.describe_target_health(TargetGroupArn=tg['TargetGroupArn'])
                    
                    for target in health_response['TargetHealthDescriptions']:
                        if target['TargetHealth']['State'] == 'healthy':
                            has_healthy_targets = True
                            break
                    
                    if has_healthy_targets:
                        break
                
                if not has_healthy_targets:
                    results.append({
                        'AccountId': session.client('sts').get_caller_identity()['Account'],
                        'Region': region,
                        'Service': self.service_name,
                        'ResourceId': lb['LoadBalancerName'],
                        'ResourceArn': lb_arn,
                        'Issue': 'Load balancer with no healthy targets',
                        'type': 'cost',
                        'Risk': 'Waste $270-360/year per LB',
                        'severity': 'high',
                        'Details': {
                            'LoadBalancerName': lb['LoadBalancerName'],
                            'Type': lb['Type'],
                            'Scheme': lb['Scheme'],
                            'State': lb['State']['Code']
                        }
                    })
        except Exception as e:
            pass
        
        return results

    def find_underutilized_lbs(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find load balancers with low request count"""
        elbv2 = session.client('elbv2', region_name=region)
        cloudwatch = session.client('cloudwatch', region_name=region)
        results = []
        
        try:
            lbs_response = elbv2.describe_load_balancers()
            end_time = datetime.utcnow()
            start_time = end_time - timedelta(days=7)
            
            for lb in lbs_response['LoadBalancers']:
                try:
                    # Get request count metrics
                    metrics_response = cloudwatch.get_metric_statistics(
                        Namespace='AWS/ApplicationELB',
                        MetricName='RequestCount',
                        Dimensions=[
                            {'Name': 'LoadBalancer', 'Value': lb['LoadBalancerArn'].split('/')[-3] + '/' + lb['LoadBalancerArn'].split('/')[-2] + '/' + lb['LoadBalancerArn'].split('/')[-1]}
                        ],
                        StartTime=start_time,
                        EndTime=end_time,
                        Period=86400,  # 1 day
                        Statistics=['Sum']
                    )
                    
                    total_requests = sum(point['Sum'] for point in metrics_response['Datapoints'])
                    avg_requests_per_day = total_requests / 7 if total_requests > 0 else 0
                    
                    if avg_requests_per_day < 100:
                        results.append({
                            'AccountId': session.client('sts').get_caller_identity()['Account'],
                            'Region': region,
                            'Service': self.service_name,
                            'ResourceId': lb['LoadBalancerName'],
                            'ResourceArn': lb['LoadBalancerArn'],
                            'Issue': 'Load balancer with <100 requests/day',
                            'type': 'cost',
                            'Risk': 'Underutilized - waste 80%+',
                            'severity': 'medium',
                            'Details': {
                                'LoadBalancerName': lb['LoadBalancerName'],
                                'Type': lb['Type'],
                                'AvgRequestsPerDay': round(avg_requests_per_day, 2)
                            }
                        })
                except Exception:
                    continue
        except Exception as e:
            pass
        
        return results

    def find_http_without_https_redirect(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find HTTP listeners without HTTPS redirect"""
        elbv2 = session.client('elbv2', region_name=region)
        results = []
        
        try:
            lbs_response = elbv2.describe_load_balancers()
            
            for lb in lbs_response['LoadBalancers']:
                if lb['Type'] != 'application':  # Only ALBs support HTTP/HTTPS
                    continue
                
                listeners_response = elbv2.describe_listeners(LoadBalancerArn=lb['LoadBalancerArn'])
                
                for listener in listeners_response['Listeners']:
                    if listener['Protocol'] == 'HTTP':
                        # Check if default action is redirect to HTTPS
                        has_https_redirect = False
                        for action in listener['DefaultActions']:
                            if (action['Type'] == 'redirect' and 
                                action.get('RedirectConfig', {}).get('Protocol') == 'HTTPS'):
                                has_https_redirect = True
                                break
                        
                        if not has_https_redirect:
                            results.append({
                                'AccountId': session.client('sts').get_caller_identity()['Account'],
                                'Region': region,
                                'Service': self.service_name,
                                'ResourceId': lb['LoadBalancerName'],
                                'ResourceArn': lb['LoadBalancerArn'],
                                'Issue': 'HTTP listener without HTTPS redirect',
                                'type': 'security',
                                'Risk': 'Man-in-the-middle attacks',
                                'severity': 'high',
                                'Details': {
                                    'LoadBalancerName': lb['LoadBalancerName'],
                                    'ListenerArn': listener['ListenerArn'],
                                    'Port': listener['Port']
                                }
                            })
        except Exception as e:
            pass
        
        return results

    def find_deprecated_tls_versions(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find load balancers with deprecated TLS versions enabled"""
        elbv2 = session.client('elbv2', region_name=region)
        results = []
        
        try:
            lbs_response = elbv2.describe_load_balancers()
            
            for lb in lbs_response['LoadBalancers']:
                if lb['Type'] != 'application':
                    continue
                
                listeners_response = elbv2.describe_listeners(LoadBalancerArn=lb['LoadBalancerArn'])
                
                for listener in listeners_response['Listeners']:
                    if listener['Protocol'] == 'HTTPS':
                        # Check SSL policy
                        ssl_policy = listener.get('SslPolicy', '')
                        
                        # Policies that support TLS 1.0/1.1
                        deprecated_policies = [
                            'ELBSecurityPolicy-2016-08',
                            'ELBSecurityPolicy-TLS-1-0-2015-04',
                            'ELBSecurityPolicy-TLS-1-1-2017-01'
                        ]
                        
                        if any(policy in ssl_policy for policy in deprecated_policies):
                            results.append({
                                'AccountId': session.client('sts').get_caller_identity()['Account'],
                                'Region': region,
                                'Service': self.service_name,
                                'ResourceId': lb['LoadBalancerName'],
                                'ResourceArn': lb['LoadBalancerArn'],
                                'Issue': 'TLS 1.0/1.1 enabled',
                                'type': 'security',
                                'Risk': 'Deprecated protocols vulnerable',
                                'severity': 'medium',
                                'Details': {
                                    'LoadBalancerName': lb['LoadBalancerName'],
                                    'ListenerArn': listener['ListenerArn'],
                                    'SslPolicy': ssl_policy
                                }
                            })
        except Exception as e:
            pass
        
        return results

    def find_lbs_without_access_logs(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find load balancers without access logs enabled"""
        elbv2 = session.client('elbv2', region_name=region)
        results = []
        
        try:
            lbs_response = elbv2.describe_load_balancers()
            
            for lb in lbs_response['LoadBalancers']:
                # Get load balancer attributes
                attrs_response = elbv2.describe_load_balancer_attributes(LoadBalancerArn=lb['LoadBalancerArn'])
                
                access_logs_enabled = False
                for attr in attrs_response['Attributes']:
                    if attr['Key'] == 'access_logs.s3.enabled' and attr['Value'] == 'true':
                        access_logs_enabled = True
                        break
                
                if not access_logs_enabled:
                    results.append({
                        'AccountId': session.client('sts').get_caller_identity()['Account'],
                        'Region': region,
                        'Service': self.service_name,
                        'ResourceId': lb['LoadBalancerName'],
                        'ResourceArn': lb['LoadBalancerArn'],
                        'Issue': 'No access logs enabled',
                        'type': 'security',
                        'Risk': 'Cannot audit traffic/attacks',
                        'severity': 'low',
                        'Details': {
                            'LoadBalancerName': lb['LoadBalancerName'],
                            'Type': lb['Type'],
                            'Scheme': lb['Scheme']
                        }
                    })
        except Exception as e:
            pass
        
        return results

    def find_classic_lbs(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find Classic Load Balancers (should use ALB/NLB)"""
        elb = session.client('elb', region_name=region)
        results = []
        
        try:
            lbs_response = elb.describe_load_balancers()
            
            for lb in lbs_response['LoadBalancerDescriptions']:
                results.append({
                    'AccountId': session.client('sts').get_caller_identity()['Account'],
                    'Region': region,
                    'Service': self.service_name,
                    'ResourceId': lb['LoadBalancerName'],
                    'ResourceArn': f"arn:aws:elasticloadbalancing:{region}:{session.client('sts').get_caller_identity()['Account']}:loadbalancer/{lb['LoadBalancerName']}",
                    'Issue': 'Classic LB (not ALB/NLB)',
                    'type': 'cost',
                    'Risk': 'Waste 10-30% vs modern LBs',
                    'severity': 'low',
                    'Details': {
                        'LoadBalancerName': lb['LoadBalancerName'],
                        'Scheme': lb['Scheme'],
                        'CreatedTime': lb['CreatedTime'].isoformat() if 'CreatedTime' in lb else None
                    }
                })
        except Exception as e:
            pass
        
        return results

    # Individual check aliases
    def check_lbs_with_no_healthy_targets(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_lbs_with_no_healthy_targets(session, region)
    
    def check_underutilized_lbs(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_underutilized_lbs(session, region)
    
    def check_http_without_https_redirect(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_http_without_https_redirect(session, region)
    
    def check_deprecated_tls_versions(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_deprecated_tls_versions(session, region)
    
    def check_lbs_without_access_logs(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_lbs_without_access_logs(session, region)
    
    def check_classic_lbs(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_classic_lbs(session, region)


import boto3
from typing import List, Dict, Any

class Route53AuditService:
    def __init__(self):
        self.service_name = "Route53"
        self.cost_checks = ['find_unused_hosted_zones']
        self.security_checks = []  # Route53 has no security checks

    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all cost-related Route53 audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results

    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all security-related Route53 audits"""
        results = []
        for check in self.security_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results

    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all Route53 audits"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results

    def find_unused_hosted_zones(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find hosted zones with only NS/SOA records"""
        route53 = session.client('route53')
        results = []
        
        try:
            # Get all hosted zones
            paginator = route53.get_paginator('list_hosted_zones')
            
            for page in paginator.paginate():
                for zone in page['HostedZones']:
                    zone_id = zone['Id'].split('/')[-1]  # Extract ID from full path
                    
                    try:
                        # Get all records for this zone
                        records_paginator = route53.get_paginator('list_resource_record_sets')
                        record_types = set()
                        
                        for records_page in records_paginator.paginate(HostedZoneId=zone_id):
                            for record in records_page['ResourceRecordSets']:
                                record_types.add(record['Type'])
                        
                        # Check if zone only has NS and SOA records
                        if record_types.issubset({'NS', 'SOA'}):
                            results.append({
                                'AccountId': session.client('sts').get_caller_identity()['Account'],
                                'Region': 'global',  # Route53 is global
                                'Service': self.service_name,
                                'ResourceId': zone_id,
                                'ResourceArn': f"arn:aws:route53:::hostedzone/{zone_id}",
                                'Issue': 'Hosted zone with only NS/SOA records',
                                'type': 'cost',
                                'Risk': 'Waste $6/mo per zone',
                                'severity': 'low',
                                'Details': {
                                    'HostedZoneId': zone_id,
                                    'Name': zone['Name'],
                                    'RecordCount': zone.get('ResourceRecordSetCount', 0),
                                    'RecordTypes': list(record_types),
                                    'Private': zone.get('Config', {}).get('PrivateZone', False)
                                }
                            })
                    except Exception:
                        continue
        except Exception as e:
            pass
        
        return results

    # Individual check aliases
    def check_unused_hosted_zones(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        return self.find_unused_hosted_zones(session, region)


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def lb(ctx):
    """Load Balancer operations"""
    pass

@lb.command('audit')
@common_options
@click.pass_context
def lb_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete Load Balancer audit"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('cost-audit')
@common_options
@click.pass_context
def lb_cost_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run Load Balancer cost optimization audit only"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('security-audit')
@common_options
@click.pass_context
def lb_security_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run Load Balancer security audit only"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('check-no-healthy-targets')
@common_options
@click.pass_context
def lb_check_no_targets(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find load balancers with no healthy targets"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'check_lbs_with_no_healthy_targets', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('check-underutilized-lbs')
@common_options
@click.pass_context
def lb_check_underutilized(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find underutilized load balancers"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'check_underutilized_lbs', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('check-classic-lbs')
@common_options
@click.pass_context
def lb_check_classic(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find Classic Load Balancers (should be ALB/NLB)"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'check_classic_lbs', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('check-http-without-https-redirect')
@common_options
@click.pass_context
def lb_check_http_redirect(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find HTTP listeners without HTTPS redirect"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'check_http_without_https_redirect', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('check-deprecated-tls-versions')
@common_options
@click.pass_context
def lb_check_tls(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find load balancers with deprecated TLS versions"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'check_deprecated_tls_versions', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@lb.command('check-lbs-without-access-logs')
@common_options
@click.pass_context
def lb_check_access_logs(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find load balancers without access logs"""
    from ..services.lb_audit import LBAuditService
    execute_service_command(ctx, LBAuditService, 'check_lbs_without_access_logs', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)


import boto3
from typing import List, Dict, Any
from datetime import datetime, timedelta

class CloudWatchAuditService:
    def __init__(self):
        self.service_name = "CloudWatch"
        self.cost_checks = ['check_log_groups_without_retention', 'check_unused_alarms', 'check_unused_custom_metrics']
        self.security_checks = []  # No security checks for CloudWatch

    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run cost-related audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run security-related audits"""
        return []  # No security checks for CloudWatch
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run complete CloudWatch audit (cost + security)"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results
    
    def check_log_groups_without_retention(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find log groups without retention policy"""
        logs = session.client('logs', region_name=region)
        results = []
        
        try:
            paginator = logs.get_paginator('describe_log_groups')
            
            for page in paginator.paginate():
                for log_group in page['logGroups']:
                    retention_days = log_group.get('retentionInDays')
                    stored_bytes = log_group.get('storedBytes', 0)
                    
                    if not retention_days:
                        monthly_cost = (stored_bytes / (1024**3)) * 0.50 if stored_bytes > 0 else 0
                        
                        results.append({
                            'AccountId': session.client('sts').get_caller_identity()['Account'],
                            'Region': region,
                            'Service': self.service_name,
                            'ResourceId': log_group['logGroupName'],
                            'ResourceName': log_group['logGroupName'],
                            'Issue': 'Log group without retention policy',
                            'type': 'cost',
                            'Risk': 'HIGH' if stored_bytes > 1024**3 else 'MEDIUM',
                            'severity': 'high' if stored_bytes > 1024**3 else 'Medium',
                            'Description': f"Log group {log_group['logGroupName']} has no retention policy and costs ${round(monthly_cost, 2)}/month",
                            'ARN': log_group['arn'],
                            'Details': {
                                'LogGroupName': log_group['logGroupName'],
                                'CreationTime': datetime.fromtimestamp(log_group['creationTime'] / 1000).isoformat(),
                                'StoredBytes': stored_bytes,
                                'StoredGB': round(stored_bytes / (1024**3), 2),
                                'EstimatedMonthlyCost': round(monthly_cost, 2)
                            }
                        })
        except Exception as e:
            print(f"Error checking CloudWatch log groups in {region}: {e}")
        
        return results

    def check_unused_alarms(self, session: boto3.Session, region: str, days: int = 30, **kwargs) -> List[Dict[str, Any]]:
        """Find unused CloudWatch alarms (no state changes in X days)"""
        cloudwatch = session.client('cloudwatch', region_name=region)
        results = []
        
        try:
            paginator = cloudwatch.get_paginator('describe_alarms')
            cutoff_date = datetime.utcnow().replace(tzinfo=None) - timedelta(days=days)
            
            for page in paginator.paginate():
                for alarm in page['MetricAlarms']:
                    state_updated = alarm.get('StateUpdatedTimestamp')
                    
                    if state_updated and state_updated.replace(tzinfo=None) < cutoff_date:
                        results.append({
                            'AccountId': session.client('sts').get_caller_identity()['Account'],
                            'Region': region,
                            'Service': self.service_name,
                            'ResourceId': alarm['AlarmName'],
                            'ResourceName': alarm['AlarmName'],
                            'Issue': 'Unused CloudWatch alarm',
                            'type': 'cost',
                            'Risk': 'LOW',
                            'severity': 'low',
                            'Description': f"CloudWatch alarm {alarm['AlarmName']} has no state changes in {days} days",
                            'ARN': alarm['AlarmArn'],
                            'Details': {
                                'AlarmName': alarm['AlarmName'],
                                'StateValue': alarm.get('StateValue'),
                                'StateUpdatedTimestamp': state_updated.isoformat() if state_updated else None,
                                'MetricName': alarm.get('MetricName')
                            }
                        })
        except Exception as e:
            print(f"Error checking CloudWatch alarms in {region}: {e}")
        
        return results

    def check_unused_custom_metrics(self, session: boto3.Session, region: str, days: int = 30, **kwargs) -> List[Dict[str, Any]]:
        """Find unused custom metrics (no data in X days)"""
       
        cloudwatch = session.client('cloudwatch', region_name=region)
        results = []
        
        try:
            paginator = cloudwatch.get_paginator('list_metrics')
            end_time = datetime.utcnow()
            start_time = end_time - timedelta(days=days)
            
            for page in paginator.paginate():
                for metric in page['Metrics']:
                    namespace = metric['Namespace']
                    
                    if namespace.startswith('AWS/'):
                        continue
                    
                    try:
                        stats_response = cloudwatch.get_metric_statistics(
                            Namespace=namespace,
                            MetricName=metric['MetricName'],
                            Dimensions=metric.get('Dimensions', []),
                            StartTime=start_time,
                            EndTime=end_time,
                            Period=86400,
                            Statistics=['SampleCount']
                        )
                        
                        if not stats_response['Datapoints']:
                            metric_name = f"{namespace}/{metric['MetricName']}"
                            results.append({
                                'AccountId': session.client('sts').get_caller_identity()['Account'],
                                'Region': region,
                                'Service': self.service_name,
                                'ResourceId': metric_name,
                                'ResourceName': metric_name,
                                'Issue': 'Unused custom metric',
                                'type': 'cost',
                                'Risk': 'LOW',
                                'severity': 'low',
                                'Description': f"Custom metric {metric_name} has no data in {days} days but costs $0.30/month",
                                'ARN': f"arn:aws:cloudwatch:{region}:{session.client('sts').get_caller_identity()['Account']}:metric/{namespace}/{metric['MetricName']}",
                                'Details': {
                                    'Namespace': namespace,
                                    'MetricName': metric['MetricName'],
                                    'Dimensions': metric.get('Dimensions', []),
                                    'EstimatedMonthlyCost': 0.30
                                }
                            })
                    except Exception:
                        continue
        except Exception as e:
            print(f"Error checking CloudWatch custom metrics in {region}: {e}")
        
        return results
    
    


import boto3
from typing import List, Dict, Any
from datetime import datetime, timedelta
import json

class IAMAuditService:
    def __init__(self):
        self.cost_checks = ['find_unused_roles']
        self.security_checks = ['find_root_access_keys', 'find_old_access_keys', 'find_inactive_users', 
                               'find_wildcard_policies', 'find_admin_no_mfa', 'find_weak_password_policy',
                               'find_no_password_rotation', 'find_cross_account_no_external_id']
    
    # Cost Audit Methods
    def find_unused_roles(self, session: boto3.Session, region: str, days: int = 90, **kwargs) -> List[Dict[str, Any]]:
        """Find unused roles creating resources"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        unused_roles = []
        
        try:
            cutoff_date = datetime.now() - timedelta(days=days)
            roles = iam.list_roles()
            
            for role in roles['Roles']:
                role_name = role['RoleName']
                
                # Skip AWS service roles
                if role_name.startswith('aws-') or role_name.startswith('AWSServiceRole'):
                    continue
                
                try:
                    # Get role last activity
                    role_details = iam.get_role(RoleName=role_name)
                    last_used = role_details['Role'].get('RoleLastUsed', {}).get('LastUsedDate')
                    
                    is_unused = False
                    if not last_used:
                        # Never used
                        is_unused = True
                    elif last_used.replace(tzinfo=None) < cutoff_date:
                        # Not used recently
                        is_unused = True
                    
                    if is_unused:
                        unused_roles.append({
                            'AccountId': account_id,
                            'Region': region,
                            'Service': 'IAM',
                            'ResourceId': role_name,
                            'ResourceName': role_name,
                            'Issue': 'Unused role creating resources',
                            'type': 'cost',
                            'Risk': 'HIGH',
                            'severity': 'high',
                            'Description': f"IAM role {role_name} unused for {days}+ days",
                            'ARN': role['Arn'],
                            'Details': {
                                'RoleName': role_name,
                                'RoleId': role['RoleId'],
                                'CreateDate': role['CreateDate'].isoformat(),
                                'LastUsed': last_used.isoformat() if last_used else 'Never'
                            }
                        })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking unused roles: {e}")
        
        return unused_roles
    
    # Security Audit Methods
    def find_root_access_keys(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find root account has access keys"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        root_keys = []
        
        try:
            # Get account summary to check for root access keys
            summary = iam.get_account_summary()
            root_access_keys = summary['SummaryMap'].get('AccountAccessKeysPresent', 0)
            
            if root_access_keys > 0:
                root_keys.append({
                    'AccountId': account_id,
                    'Region': region,
                    'Service': 'IAM',
                    'ResourceId': 'root',
                    'ResourceName': 'root',
                    'Issue': 'Root account has access keys',
                    'type': 'security',
                    'Risk': 'CRITICAL',
                    'severity': 'critical',
                    'Description': 'Root account has active access keys - immediate security risk',
                    'ARN': f'arn:aws:iam::{account_id}:root',
                    'Details': {
                        'AccessKeysCount': root_access_keys
                    }
                })
        except Exception as e:
            print(f"Error checking root access keys: {e}")
        
        return root_keys
    
    def find_old_access_keys(self, session: boto3.Session, region: str, days: int = 90, **kwargs) -> List[Dict[str, Any]]:
        """Find access keys older than 90 days"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        old_keys = []
        
        try:
            cutoff_date = datetime.now() - timedelta(days=days)
            users = iam.list_users()
            
            for user in users['Users']:
                user_name = user['UserName']
                
                try:
                    access_keys = iam.list_access_keys(UserName=user_name)
                    
                    for key in access_keys['AccessKeyMetadata']:
                        key_age = (datetime.now() - key['CreateDate'].replace(tzinfo=None)).days
                        
                        if key_age > days:
                            old_keys.append({
                                'AccountId': account_id,
                                'Region': region,
                                'Service': 'IAM',
                                'ResourceId': user_name,
                                'ResourceName': user_name,
                                'Issue': f'Access keys older than {days} days',
                                'type': 'security',
                                'Risk': 'CRITICAL',
                                'severity': 'critical',
                                'Description': f"User {user_name} has access key aged {key_age} days",
                                'ARN': user['Arn'],
                                'Details': {
                                    'UserName': user_name,
                                    'AccessKeyId': key['AccessKeyId'],
                                    'CreateDate': key['CreateDate'].isoformat(),
                                    'Age': key_age,
                                    'Status': key['Status']
                                }
                            })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking old access keys: {e}")
        
        return old_keys
    
    def find_inactive_users(self, session: boto3.Session, region: str, days: int = 90, **kwargs) -> List[Dict[str, Any]]:
        """Find users inactive >90 days with active keys"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        inactive_users = []
        
        try:
            cutoff_date = datetime.now() - timedelta(days=days)
            users = iam.list_users()
            
            for user in users['Users']:
                user_name = user['UserName']
                
                try:
                    # Check if user has active access keys
                    access_keys = iam.list_access_keys(UserName=user_name)
                    has_active_keys = any(key['Status'] == 'Active' for key in access_keys['AccessKeyMetadata'])
                    
                    if not has_active_keys:
                        continue
                    
                    # Get user last activity
                    user_details = iam.get_user(UserName=user_name)
                    password_last_used = user_details['User'].get('PasswordLastUsed')
                    
                    # Check access key last used
                    last_activity = None
                    for key in access_keys['AccessKeyMetadata']:
                        try:
                            key_last_used = iam.get_access_key_last_used(AccessKeyId=key['AccessKeyId'])
                            key_last_activity = key_last_used.get('AccessKeyLastUsed', {}).get('LastUsedDate')
                            if key_last_activity:
                                if not last_activity or key_last_activity > last_activity:
                                    last_activity = key_last_activity
                        except Exception:
                            continue
                    
                    # Compare with password last used
                    if password_last_used:
                        if not last_activity or password_last_used > last_activity:
                            last_activity = password_last_used
                    
                    # Check if inactive
                    is_inactive = False
                    if not last_activity:
                        is_inactive = True
                    elif last_activity.replace(tzinfo=None) < cutoff_date:
                        is_inactive = True
                    
                    if is_inactive:
                        inactive_users.append({
                            'AccountId': account_id,
                            'Region': region,
                            'Service': 'IAM',
                            'ResourceId': user_name,
                            'ResourceName': user_name,
                            'Issue': f'User inactive >{days} days with active keys',
                            'type': 'security',
                            'Risk': 'HIGH',
                            'severity': 'high',
                            'Description': f"User {user_name} inactive for {days}+ days but has active access keys",
                            'ARN': user['Arn'],
                            'Details': {
                                'UserName': user_name,
                                'CreateDate': user['CreateDate'].isoformat(),
                                'LastActivity': last_activity.isoformat() if last_activity else 'Never',
                                'ActiveKeys': len([k for k in access_keys['AccessKeyMetadata'] if k['Status'] == 'Active'])
                            }
                        })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking inactive users: {e}")
        
        return inactive_users
    
    def find_wildcard_policies(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find policies using Action:* or Resource:*"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        wildcard_policies = []
        
        try:
            # Check user policies
            users = iam.list_users()
            for user in users['Users']:
                user_name = user['UserName']
                
                try:
                    # Check inline policies
                    inline_policies = iam.list_user_policies(UserName=user_name)
                    for policy_name in inline_policies['PolicyNames']:
                        policy = iam.get_user_policy(UserName=user_name, PolicyName=policy_name)
                        if self._has_wildcard_permissions(policy['PolicyDocument']):
                            wildcard_policies.append({
                                'AccountId': account_id,
                                'Type': 'User',
                                'Name': user_name,
                                'PolicyName': policy_name,
                                'PolicyType': 'Inline',
                                'ARN': user['Arn'],
                                'Region': region,
                                'Issue': 'Policy uses Action:* or Resource:*',
                                'type': 'security',
                                'Risk': 'Privilege escalation risk',
                                'severity': 'high',
                                'Service': 'IAM'
                            })
                    
                    # Check attached policies
                    attached_policies = iam.list_attached_user_policies(UserName=user_name)
                    for policy in attached_policies['AttachedPolicies']:
                        if policy['PolicyArn'].startswith('arn:aws:iam::aws:'):
                            continue  # Skip AWS managed policies
                        
                        try:
                            policy_version = iam.get_policy(PolicyArn=policy['PolicyArn'])
                            policy_doc = iam.get_policy_version(
                                PolicyArn=policy['PolicyArn'],
                                VersionId=policy_version['Policy']['DefaultVersionId']
                            )
                            if self._has_wildcard_permissions(policy_doc['PolicyVersion']['Document']):
                                wildcard_policies.append({
                                    'AccountId': account_id,
                                    'Type': 'User',
                                    'Name': user_name,
                                    'PolicyName': policy['PolicyName'],
                                    'PolicyType': 'Managed',
                                    'ARN': user['Arn'],
                                    'Region': region,
                                    'Issue': 'Policy uses Action:* or Resource:*',
                                    'type': 'security',
                                    'Risk': 'Privilege escalation risk',
                                    'severity': 'high',
                                    'Service': 'IAM'
                                })
                        except Exception:
                            continue
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking wildcard policies: {e}")
        
        return wildcard_policies
    
    def _has_wildcard_permissions(self, policy_document):
        """Check if policy document has wildcard permissions"""
        if isinstance(policy_document, str):
            policy_document = json.loads(policy_document)
        
        statements = policy_document.get('Statement', [])
        if not isinstance(statements, list):
            statements = [statements]
        
        for statement in statements:
            if statement.get('Effect') != 'Allow':
                continue
            
            actions = statement.get('Action', [])
            resources = statement.get('Resource', [])
            
            if not isinstance(actions, list):
                actions = [actions]
            if not isinstance(resources, list):
                resources = [resources]
            
            # Check for wildcard actions or resources
            if '*' in actions or '*' in resources:
                return True
        
        return False
    
    def find_admin_no_mfa(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find admin access without MFA"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        admin_no_mfa = []
        
        try:
            users = iam.list_users()
            
            for user in users['Users']:
                user_name = user['UserName']
                
                try:
                    # Check if user has admin permissions
                    has_admin = False
                    
                    # Check attached policies
                    attached_policies = iam.list_attached_user_policies(UserName=user_name)
                    for policy in attached_policies['AttachedPolicies']:
                        if 'Administrator' in policy['PolicyName'] or policy['PolicyArn'].endswith('AdministratorAccess'):
                            has_admin = True
                            break
                    
                    # Check groups
                    if not has_admin:
                        groups = iam.get_groups_for_user(UserName=user_name)
                        for group in groups['Groups']:
                            group_policies = iam.list_attached_group_policies(GroupName=group['GroupName'])
                            for policy in group_policies['AttachedPolicies']:
                                if 'Administrator' in policy['PolicyName'] or policy['PolicyArn'].endswith('AdministratorAccess'):
                                    has_admin = True
                                    break
                            if has_admin:
                                break
                    
                    if has_admin:
                        # Check if user has MFA
                        mfa_devices = iam.list_mfa_devices(UserName=user_name)
                        if not mfa_devices['MFADevices']:
                            admin_no_mfa.append({
                                'AccountId': account_id,
                                'UserName': user_name,
                                'ARN': user['Arn'],
                                'Region': region,
                                'CreateDate': user['CreateDate'].isoformat(),
                                'MFADevices': len(mfa_devices['MFADevices']),
                                'Issue': 'Admin access without MFA',
                                'type': 'security',
                                'Risk': 'Account takeover via phishing',
                                'severity': 'high',
                                'Service': 'IAM'
                            })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking admin without MFA: {e}")
        
        return admin_no_mfa
    
    def find_weak_password_policy(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find password policy allows weak passwords"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        weak_policy = []
        
        try:
            try:
                policy = iam.get_account_password_policy()['PasswordPolicy']
            except iam.exceptions.NoSuchEntityException:
                # No password policy exists
                weak_policy.append({
                    'AccountId': account_id,
                    'Region': region,
                    'Service': 'IAM',
                    'ResourceId': 'password-policy',
                    'ResourceName': 'Password Policy',
                    'Issue': 'Password policy allows weak passwords',
                    'type': 'security',
                    'Risk': 'MEDIUM',
                    'severity': 'medium',
                    'Description': 'No password policy configured - allows weak passwords',
                    'ARN': f'arn:aws:iam::{account_id}:account-password-policy',
                    'Details': {
                        'PolicyStatus': 'No password policy configured'
                    }
                })
                return weak_policy
            
            # Check for weak settings
            weak_settings = []
            
            if policy.get('MinimumPasswordLength', 0) < 8:
                weak_settings.append(f"Min length: {policy.get('MinimumPasswordLength', 0)} < 8")
            
            if not policy.get('RequireUppercaseCharacters', False):
                weak_settings.append("No uppercase required")
            
            if not policy.get('RequireLowercaseCharacters', False):
                weak_settings.append("No lowercase required")
            
            if not policy.get('RequireNumbers', False):
                weak_settings.append("No numbers required")
            
            if not policy.get('RequireSymbols', False):
                weak_settings.append("No symbols required")
            
            if weak_settings:
                weak_policy.append({
                    'AccountId': account_id,
                    'Region': region,
                    'Service': 'IAM',
                    'ResourceId': 'password-policy',
                    'ResourceName': 'Password Policy',
                    'Issue': 'Password policy allows weak passwords',
                    'type': 'security',
                    'Risk': 'MEDIUM',
                    'severity': 'medium',
                    'Description': f"Password policy has weak settings: {', '.join(weak_settings)}",
                    'ARN': f'arn:aws:iam::{account_id}:account-password-policy',
                    'Details': {
                        'WeakSettings': weak_settings
                    }
                })
        except Exception as e:
            print(f"Error checking password policy: {e}")
        
        return weak_policy
    
    def find_no_password_rotation(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find no password rotation policy"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        no_rotation = []
        
        try:
            try:
                policy = iam.get_account_password_policy()['PasswordPolicy']
                max_age = policy.get('MaxPasswordAge')
                
                if not max_age or max_age > 90:
                    no_rotation.append({
                        'AccountId': account_id,
                        'Region': region,
                        'Service': 'IAM',
                        'ResourceId': 'password-policy',
                        'ResourceName': 'Password Policy',
                        'Issue': 'No password rotation policy',
                        'type': 'security',
                        'Risk': 'MEDIUM',
                        'severity': 'medium',
                        'Description': f"Password rotation not enforced (max age: {max_age or 'Not set'})",
                        'ARN': f'arn:aws:iam::{account_id}:account-password-policy',
                        'Details': {
                            'MaxPasswordAge': max_age or 'Not set'
                        }
                    })
            except iam.exceptions.NoSuchEntityException:
                no_rotation.append({
                    'AccountId': account_id,
                    'Region': region,
                    'Service': 'IAM',
                    'ResourceId': 'password-policy',
                    'ResourceName': 'Password Policy',
                    'Issue': 'No password rotation policy',
                    'type': 'security',
                    'Risk': 'MEDIUM',
                    'severity': 'medium',
                    'Description': 'No password policy configured - no rotation enforced',
                    'ARN': f'arn:aws:iam::{account_id}:account-password-policy',
                    'Details': {
                        'MaxPasswordAge': 'No policy'
                    }
                })
        except Exception as e:
            print(f"Error checking password rotation: {e}")
        
        return no_rotation
    
    def find_cross_account_no_external_id(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find cross-account roles without ExternalId"""
        iam = session.client('iam')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        no_external_id = []
        
        try:
            roles = iam.list_roles()
            
            for role in roles['Roles']:
                role_name = role['RoleName']
                
                try:
                    assume_role_policy = role['AssumeRolePolicyDocument']
                    if isinstance(assume_role_policy, str):
                        assume_role_policy = json.loads(assume_role_policy)
                    
                    statements = assume_role_policy.get('Statement', [])
                    if not isinstance(statements, list):
                        statements = [statements]
                    
                    for statement in statements:
                        principal = statement.get('Principal', {})
                        if isinstance(principal, dict) and 'AWS' in principal:
                            aws_principals = principal['AWS']
                            if not isinstance(aws_principals, list):
                                aws_principals = [aws_principals]
                            
                            # Check for cross-account principals
                            for aws_principal in aws_principals:
                                if ':' in aws_principal and account_id not in aws_principal:
                                    # This is a cross-account role
                                    condition = statement.get('Condition', {})
                                    has_external_id = any('ExternalId' in str(cond) for cond in condition.values()) if condition else False
                                    
                                    if not has_external_id:
                                        no_external_id.append({
                                            'AccountId': account_id,
                                            'RoleName': role_name,
                                            'ARN': role['Arn'],
                                            'Region': region,
                                            'CrossAccountPrincipal': aws_principal,
                                            'Issue': 'Cross-account role without ExternalId',
                                            'type': 'security',
                                            'Risk': 'Confused deputy attack',
                                            'severity': 'high',
                                            'Service': 'IAM'
                                        })
                                        break
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking cross-account roles: {e}")
        
        return no_external_id
    
    # Audit Methods
    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all cost-related audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all security-related audits"""
        results = []
        for check in self.security_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all audits (cost + security)"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results
    
    # Individual Check Method Aliases
    def check_unused_roles(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_unused_roles(session, region, **kwargs)
    
    def check_root_access_keys(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_root_access_keys(session, region, **kwargs)
    
    def check_old_access_keys(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_old_access_keys(session, region, **kwargs)
    
    def check_inactive_users(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_inactive_users(session, region, **kwargs)
    
    def check_wildcard_policies(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_wildcard_policies(session, region, **kwargs)
    
    def check_admin_no_mfa(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_admin_no_mfa(session, region, **kwargs)
    
    def check_weak_password_policy(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_weak_password_policy(session, region, **kwargs)
    
    def check_no_password_rotation(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_no_password_rotation(session, region, **kwargs)
    
    def check_cross_account_no_external_id(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_cross_account_no_external_id(session, region, **kwargs)


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def cloudwatch(ctx):
    """CloudWatch operations"""
    pass

@cloudwatch.command('audit')
@click.option('--days', default=30, help='Days threshold for unused resources')
@common_options
@click.pass_context
def cloudwatch_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete CloudWatch audit"""
    from ..services.cloudwatch_audit import CloudWatchAuditService
    execute_service_command(ctx, CloudWatchAuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@cloudwatch.command('cost-audit')
@click.option('--days', default=30, help='Days threshold for unused resources')
@common_options
@click.pass_context
def cloudwatch_cost_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run CloudWatch cost optimization audit only"""
    from ..services.cloudwatch_audit import CloudWatchAuditService
    execute_service_command(ctx, CloudWatchAuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@cloudwatch.command('security-audit')
@common_options
@click.pass_context
def cloudwatch_security_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run CloudWatch security audit only"""
    from ..services.cloudwatch_audit import CloudWatchAuditService
    execute_service_command(ctx, CloudWatchAuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@cloudwatch.command('check-unused-alarms')
@click.option('--days', default=30, help='Days threshold for alarm activity')
@common_options
@click.pass_context
def cloudwatch_check_alarms(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find unused CloudWatch alarms"""
    from ..services.cloudwatch_audit import CloudWatchAuditService
    execute_service_command(ctx, CloudWatchAuditService, 'check_unused_alarms', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@cloudwatch.command('check-log-retention')
@common_options
@click.pass_context
def cloudwatch_check_logs(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find log groups without retention policies"""
    from ..services.cloudwatch_audit import CloudWatchAuditService
    execute_service_command(ctx, CloudWatchAuditService, 'check_log_groups_without_retention', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@cloudwatch.command('check_unused_custom_metrics')
@click.option('--days', default=30, help='Days threshold for metrics activity')
@common_options
@click.pass_context
def cloudwatch_check_custom_metrics(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find unused custom metrics (no data in X days)"""
    from ..services.cloudwatch_audit import CloudWatchAuditService
    execute_service_command(ctx, CloudWatchAuditService, 'check_unused_custom_metrics', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)


import boto3
from typing import List, Dict, Any
from concurrent.futures import ThreadPoolExecutor, as_completed

class EIPAuditService:
    def __init__(self):
        self.service_name = "EIP"
        self.cost_checks = ['check_unattached_eips', 'check_eips_on_stopped_instances']
        self.security_checks = ['check_eips_with_dangerous_sg_rules']

    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run cost-related audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run security-related audits"""
        results = []
        for check in self.security_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run complete EIP audit (cost + security)"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results
    
    
    def check_unattached_eips(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find unattached Elastic IPs"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            response = ec2.describe_addresses()
            for eip in response['Addresses']:
                if 'InstanceId' not in eip and 'NetworkInterfaceId' not in eip:
                    results.append({
                        'AccountId': session.client('sts').get_caller_identity()['Account'],
                        'Region': region,
                        'Service': self.service_name,
                        'ResourceId': eip.get('AllocationId', eip.get('PublicIp')),
                        'ResourceName': eip.get('PublicIp'),
                        'Issue': 'Unattached Elastic IP',
                        'type': 'cost',
                        'Risk': 'HIGH',
                        'severity': 'high',
                        'Description': f"Elastic IP {eip.get('PublicIp')} is unattached and costs $43.80/month",
                        'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:elastic-ip/{eip.get('AllocationId', eip.get('PublicIp'))}",
                        'Details': {
                            'PublicIp': eip.get('PublicIp'),
                            'AllocationId': eip.get('AllocationId'),
                            'Domain': eip.get('Domain')
                        }
                    })
        except Exception as e:
            print(f"Error checking unattached EIPs in {region}: {e}")
        
        return results

    def check_eips_on_stopped_instances(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find EIPs attached to stopped instances"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            eips_response = ec2.describe_addresses()
            instance_ids = [eip['InstanceId'] for eip in eips_response['Addresses'] if 'InstanceId' in eip]
            
            if instance_ids:
                instances_response = ec2.describe_instances(InstanceIds=instance_ids)
                stopped_instances = set()
                
                for reservation in instances_response['Reservations']:
                    for instance in reservation['Instances']:
                        if instance['State']['Name'] == 'stopped':
                            stopped_instances.add(instance['InstanceId'])
                
                for eip in eips_response['Addresses']:
                    if 'InstanceId' in eip and eip['InstanceId'] in stopped_instances:
                        results.append({
                            'AccountId': session.client('sts').get_caller_identity()['Account'],
                            'Region': region,
                            'Service': self.service_name,
                            'ResourceId': eip.get('AllocationId', eip.get('PublicIp')),
                            'ResourceName': eip.get('PublicIp'),
                            'Issue': 'EIP attached to stopped instance',
                            'type': 'cost',
                            'Risk': 'HIGH',
                            'severity': 'high',
                            'Description': f"Elastic IP {eip.get('PublicIp')} attached to stopped instance {eip['InstanceId']}",
                            'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:elastic-ip/{eip.get('AllocationId', eip.get('PublicIp'))}",
                            'Details': {
                                'PublicIp': eip.get('PublicIp'),
                                'AllocationId': eip.get('AllocationId'),
                                'InstanceId': eip['InstanceId']
                            }
                        })
        except Exception as e:
            print(f"Error checking EIPs on stopped instances in {region}: {e}")
        
        return results

    def check_eips_with_dangerous_sg_rules(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Find EIPs on instances with dangerous security group rules"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            eips_response = ec2.describe_addresses()
            instance_eips = {eip['InstanceId']: eip for eip in eips_response['Addresses'] if 'InstanceId' in eip}
            
            if instance_eips:
                instances_response = ec2.describe_instances(InstanceIds=list(instance_eips.keys()))
                
                for reservation in instances_response['Reservations']:
                    for instance in reservation['Instances']:
                        instance_id = instance['InstanceId']
                        sg_ids = [sg['GroupId'] for sg in instance['SecurityGroups']]
                        
                        sgs_response = ec2.describe_security_groups(GroupIds=sg_ids)
                        has_dangerous_rule = False
                        
                        for sg in sgs_response['SecurityGroups']:
                            for rule in sg.get('IpPermissions', []):
                                for ip_range in rule.get('IpRanges', []):
                                    if ip_range.get('CidrIp') == '0.0.0.0/0':
                                        from_port = rule.get('FromPort', 0)
                                        to_port = rule.get('ToPort', 65535)
                                        
                                        dangerous_ports = [22, 3389, 3306, 5432, 1433, 27017]
                                        if any(port >= from_port and port <= to_port for port in dangerous_ports):
                                            has_dangerous_rule = True
                                            break
                                if has_dangerous_rule:
                                    break
                            if has_dangerous_rule:
                                break
                        
                        if has_dangerous_rule:
                            eip = instance_eips[instance_id]
                            results.append({
                                'AccountId': session.client('sts').get_caller_identity()['Account'],
                                'Region': region,
                                'Service': self.service_name,
                                'ResourceId': eip.get('AllocationId', eip.get('PublicIp')),
                                'ResourceName': eip.get('PublicIp'),
                                'Issue': 'EIP on instance with dangerous SG rules',
                                'type': 'security',
                                'Risk': 'HIGH',
                                'severity': 'high',
                                'Description': f"Elastic IP {eip.get('PublicIp')} on instance with dangerous security group rules",
                                'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:elastic-ip/{eip.get('AllocationId', eip.get('PublicIp'))}",
                                'Details': {
                                    'PublicIp': eip.get('PublicIp'),
                                    'AllocationId': eip.get('AllocationId'),
                                    'InstanceId': instance_id,
                                    'SecurityGroups': sg_ids
                                }
                            })
        except Exception as e:
            print(f"Error checking EIPs with dangerous SG rules in {region}: {e}")
        
        return results
    
    


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def iam(ctx):
    """IAM operations"""
    pass

@iam.command('audit')
@click.option('--days', default=90, help='Days threshold for unused roles, inactive users and old keys')
@common_options
@click.pass_context
def iam_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete IAM audit (cost + security)"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@iam.command('security-audit')
@click.option('--days', default=90, help='Days threshold for unused roles, inactive users and old keys')
@common_options
@click.pass_context
def iam_security_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run IAM security audit only"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@iam.command('check-root-access-keys')
@common_options
@click.pass_context
def iam_check_root_keys(ctx, organization, region, max_workers, regions, output):
    """Find root account access keys"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_root_access_keys', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@iam.command('cost-audit')
@click.option('--days', default=90, help='Days threshold for unused roles')
@common_options
@click.pass_context
def iam_cost_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run IAM cost optimization audit only"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@iam.command('check-unused-roles')
@click.option('--days', default=90, help='Days threshold for unused roles')
@common_options
@click.pass_context
def iam_check_unused_roles(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find unused IAM roles"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_unused_roles', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@iam.command('check-inactive-users')
@click.option('--days', default=90, help='Days threshold for inactive users')
@common_options
@click.pass_context
def iam_check_inactive_users(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find inactive IAM users"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_inactive_users', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@iam.command('check-old-access-keys')
@click.option('--days', default=90, help='Days threshold for old access keys')
@common_options
@click.pass_context
def iam_check_old_keys(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find old IAM access keys"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_old_access_keys', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@iam.command('check-wildcard-policies')
@common_options
@click.pass_context
def iam_check_wildcard_policies(ctx, organization, region, max_workers, regions, output):
    """Find IAM policies with wildcard permissions"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_wildcard_policies', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@iam.command('check-admin-no-mfa')
@common_options
@click.pass_context
def iam_check_admin_no_mfa(ctx, organization, region, max_workers, regions, output):
    """Find admin users without MFA"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_admin_no_mfa', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@iam.command('check-weak-password-policy')
@common_options
@click.pass_context
def iam_check_weak_password(ctx, organization, region, max_workers, regions, output):
    """Find weak password policy"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_weak_password_policy', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@iam.command('check-no-password-rotation')
@common_options
@click.pass_context
def iam_check_no_rotation(ctx, organization, region, max_workers, regions, output):
    """Find users with no password rotation"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_no_password_rotation', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@iam.command('check-cross-account-no-external-id')
@common_options
@click.pass_context
def iam_check_cross_account(ctx, organization, region, max_workers, regions, output):
    """Find cross-account roles without external ID"""
    from ..services.iam_audit import IAMAuditService
    execute_service_command(ctx, IAMAuditService, 'check_cross_account_no_external_id', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)


import click
import asyncio
from ..core.executor import ServiceExecutor

def common_options(f):
    """Decorator to add common options to all service commands"""
    f = click.option('--output', default='console', type=click.Choice(['console', 'json', 'csv']), help='Output format')(f)
    f = click.option('--save-to', help='Save output to S3 (s3://bucket/path) or local path (/path/to/file)')(f)
    f = click.option('--regions', help='Comma-separated list of regions (e.g., us-east-1,eu-west-1)')(f)
    f = click.option('--max-workers', default=10, help='Maximum number of worker threads')(f)
    f = click.option('--region', help='AWS region to scan')(f)
    f = click.option('--organization', is_flag=True, help='Scan entire AWS organization')(f)
    f = click.option('--cross-account-role', default='OrganizationAccountAccessRole', help='Role name for cross-account access')(f)
    f = click.option('--org-admin-account-id', help='Organization admin account ID (if different from current account)')(f)
    return f

def get_effective_params(ctx, organization, region, max_workers, regions=None, cross_account_role=None, org_admin_account_id=None):
    """Get effective parameters, preferring command-level over global"""
    # Priority: regions > region > global region
    effective_regions = None
    if regions:
        effective_regions = [r.strip() for r in regions.split(',')]
    elif region:
        effective_regions = [region]
    elif ctx.obj['region']:
        effective_regions = [ctx.obj['region']]
    else:
        effective_regions = ['us-east-1']
    
    return (
        organization or ctx.obj['organization'],
        effective_regions,
        max_workers or ctx.obj['max_workers'],
        cross_account_role or 'OrganizationAccountAccessRole',
        org_admin_account_id
    )

def execute_service_command(ctx, service_class, method, output, organization, region, max_workers, regions, cross_account_role=None, org_admin_account_id=None, save_to=None, **kwargs):
    """Execute a service command with common parameters"""
    org, reg_list, workers, role_name, admin_account = get_effective_params(ctx, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)
    service = service_class()
    executor = ServiceExecutor(service, org, reg_list, workers, role_name, admin_account)
    asyncio.run(executor.execute(method, output, save_to=save_to, **kwargs))


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def backup(ctx):
    """AWS Backup operations"""
    pass

@backup.command('audit')
@common_options
@click.pass_context
def backup_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete AWS Backup audit"""
    from ..services.backup_audit import BackupAuditService
    execute_service_command(ctx, BackupAuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@backup.command('cost-audit')
@common_options
@click.pass_context
def backup_cost_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run AWS Backup cost optimization audit only"""
    from ..services.backup_audit import BackupAuditService
    execute_service_command(ctx, BackupAuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@backup.command('security-audit')
@common_options
@click.pass_context
def backup_security_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run AWS Backup security audit only"""
    from ..services.backup_audit import BackupAuditService
    execute_service_command(ctx, BackupAuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@backup.command('check-empty-vaults')
@common_options
@click.pass_context
def backup_check_empty(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find empty backup vaults"""
    from ..services.backup_audit import BackupAuditService
    execute_service_command(ctx, BackupAuditService, 'check_empty_backup_vaults', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@backup.command('check-cross-region-backup')
@common_options
@click.pass_context
def backup_check_cross_region(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find cross-region backup for dev/test"""
    from ..services.backup_audit import BackupAuditService
    execute_service_command(ctx, BackupAuditService, 'check_cross_region_backup_dev_test', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)


import boto3
from typing import List, Dict, Any
from datetime import datetime, timedelta

class APIGatewayAuditService:
    def __init__(self):
        self.service_name = "APIGateway"
        self.cost_checks = ['check_unused_apis']
        self.security_checks = []  # No security checks for API Gateway
    
    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all cost-related API Gateway audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region, **kwargs))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all security-related API Gateway audits"""
        # No security checks for API Gateway
        return []
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all API Gateway audits (cost + security)"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results
    
    def check_unused_apis(self, session: boto3.Session, region: str, days: int = 30, **kwargs) -> List[Dict[str, Any]]:
        """Find API Gateway APIs with no requests"""
        apigateway = session.client('apigateway', region_name=region)
        cloudwatch = session.client('cloudwatch', region_name=region)
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        results = []
        
        try:
            response = apigateway.get_rest_apis()
            end_time = datetime.utcnow()
            start_time = end_time - timedelta(days=days)
            
            for api in response['items']:
                try:
                    # Check API request count
                    metrics = cloudwatch.get_metric_statistics(
                        Namespace='AWS/ApiGateway',
                        MetricName='Count',
                        Dimensions=[
                            {'Name': 'ApiName', 'Value': api['name']}
                        ],
                        StartTime=start_time,
                        EndTime=end_time,
                        Period=86400,
                        Statistics=['Sum']
                    )
                    
                    total_requests = sum(dp['Sum'] for dp in metrics['Datapoints']) if metrics['Datapoints'] else 0
                    
                    if total_requests == 0:
                        results.append({
                            'AccountId': account_id,
                            'Region': region,
                            'Service': self.service_name,
                            'ResourceId': api['id'],
                            'ResourceArn': f"arn:aws:apigateway:{region}::/restapis/{api['id']}",
                            'Issue': f'API unused (0 requests {days} days)',
                            'type': 'cost',
                            'Risk': 'Waste $3.50/mo per API',
                            'severity': 'low',
                            'Details': {
                                'ApiId': api['id'],
                                'ApiName': api['name'],
                                'CreatedDate': api['createdDate'].isoformat(),
                                'Description': api.get('description', 'N/A'),
                                'EndpointConfiguration': api.get('endpointConfiguration', {}).get('types', [])
                            }
                        })
                except Exception:
                    continue
        except Exception as e:
            pass
        
        return results
    
   


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def ebs(ctx):
    """EBS volume operations"""
    pass

@ebs.command('audit')
@click.option('--days', default=7, help='Days threshold for I/O analysis and snapshots')
@common_options
@click.pass_context
def ebs_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete EBS audit (cost + security)"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@ebs.command('cost-audit')
@click.option('--days', default=7, help='Days threshold for I/O analysis')
@common_options
@click.pass_context
def ebs_cost_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run EBS cost optimization audit only"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@ebs.command('security-audit')
@common_options
@click.pass_context
def ebs_security_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run EBS security audit only"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

# Cost optimization checks
@ebs.command('check-orphan-volumes')
@common_options
@click.pass_context
def ebs_check_orphan(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find orphaned EBS volumes"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_orphan_volumes', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@ebs.command('check-low-io-volumes')
@click.option('--iops-threshold', default=10, help='IOPS per GB threshold')
@click.option('--days', default=7, help='Days to analyze I/O')
@common_options
@click.pass_context
def ebs_check_low_io(ctx, iops_threshold, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find volumes with low I/O"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_low_io_volumes', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, iops_threshold=iops_threshold, days=days)

@ebs.command('check-old-snapshots')
@click.option('--days', default=90, help='Days threshold for old snapshots')
@common_options
@click.pass_context
def ebs_check_old_snapshots(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find old EBS snapshots"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_old_snapshots', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@ebs.command('check-gp2-volumes')
@common_options
@click.pass_context
def ebs_check_gp2(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find gp2 volumes (should be gp3)"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_gp2_volumes', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

# Security checks
@ebs.command('check-unencrypted-orphan')
@common_options
@click.pass_context
def ebs_check_unencrypted_orphan(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find unencrypted orphaned volumes"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_unencrypted_orphan', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@ebs.command('check-unencrypted-in-use')
@common_options
@click.pass_context
def ebs_check_unencrypted_in_use(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find unencrypted volumes in use"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_unencrypted_in_use', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@ebs.command('check-public-snapshots')
@common_options
@click.pass_context
def ebs_check_public_snapshots(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find public snapshots"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_public_snapshots', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@ebs.command('check-no-recent-snapshot')
@click.option('--days', default=7, help='Days threshold for recent snapshots')
@common_options
@click.pass_context
def ebs_check_no_recent_snapshot(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find volumes with no recent snapshots"""
    from ..services.ebs_audit import EBSAuditService
    execute_service_command(ctx, EBSAuditService, 'check_no_recent_snapshot', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)


import sys
from typing import Optional
import threading
import time

class ProgressBar:
    def __init__(self, total: int, description: str = "Processing", width: int = 50):
        self.total = total
        self.current = 0
        self.description = description
        self.width = width
        self.start_time = time.time()
        self._lock = threading.Lock()
        
    def update(self, increment: int = 1):
        with self._lock:
            self.current += increment
            self._display()
    
    def set_description(self, description: str):
        with self._lock:
            self.description = description
            self._display()
    
    def _display(self):
        if self.total == 0:
            return
            
        percent = min(100, (self.current / self.total) * 100)
        filled = int(self.width * self.current // self.total)
        bar = '█' * filled + '░' * (self.width - filled)
        
        elapsed = time.time() - self.start_time
        if self.current > 0:
            eta = (elapsed / self.current) * (self.total - self.current)
            eta_str = f" ETA: {int(eta)}s" if eta > 1 else ""
        else:
            eta_str = ""
        
        # Clear line and write progress
        sys.stdout.write('\r' + ' ' * 100 + '\r')
        sys.stdout.write(f'{self.description}: |{bar}| {percent:.1f}% ({self.current}/{self.total}){eta_str}')
        sys.stdout.flush()
        
        if self.current >= self.total:
            sys.stdout.write('\r' + ' ' * 100 + '\r')
            sys.stdout.flush()

class SpinnerProgress:
    def __init__(self, description: str = "Processing"):
        self.description = description
        self.spinning = False
        self.spinner_chars = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏']
        self.current_char = 0
        self._thread = None
        
    def start(self):
        self.spinning = True
        self._thread = threading.Thread(target=self._spin)
        self._thread.daemon = True
        self._thread.start()
        
    def stop(self):
        self.spinning = False
        if self._thread:
            self._thread.join()
        sys.stdout.write('\r' + ' ' * 100 + '\r')
        sys.stdout.flush()
        
    def _spin(self):
        while self.spinning:
            char = self.spinner_chars[self.current_char]
            sys.stdout.write(f'\r{char} {self.description}...')
            sys.stdout.flush()
            self.current_char = (self.current_char + 1) % len(self.spinner_chars)
            time.sleep(0.1)


import asyncio
import boto3
import json
from concurrent.futures import ThreadPoolExecutor
from typing import List, Dict, Any, Optional
from datetime import datetime
from .progress import ProgressBar, SpinnerProgress
from .storage import StorageManager

class ServiceExecutor:
    def __init__(self, service, organization: bool, regions: List[str], max_workers: int = 5, cross_account_role: str = 'OrganizationAccountAccessRole', org_admin_account_id: str = None):
        self.service = service
        self.organization = organization
        self.regions = regions
        self.max_workers = max_workers
        self.cross_account_role = cross_account_role
        self.org_admin_account_id = org_admin_account_id
        self.storage_manager = StorageManager()
        
    def _generate_filename(self, method_name: str, results: Dict[str, Any], file_format: str) -> str:
        """Generate descriptive filename based on scan parameters"""
        timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
        
        # Determine service name from class name
        service_name = getattr(self.service, '__class__', type(self.service)).__name__.lower()
        if 'auditservice' in service_name:
            service_name = service_name.replace('auditservice', '')
        
        # Determine scope
        if self.organization:
            if self.org_admin_account_id:
                scope = f"org_{self.org_admin_account_id}"
            else:
                scope = "org"
        else:
            # Single account - get account ID from results
            account_id = list(results.keys())[0] if results else "unknown"
            scope = account_id
        
        # Build filename components
        parts = ['kosty']
        
        # Add service or "full" for comprehensive scans
        if service_name and service_name != 'comprehensive':
            parts.append(service_name)
        else:
            parts.append('full')
        
        # Add method if not standard audit
        if method_name != 'audit':
            parts.append(method_name)
        
        # Add scope
        parts.append(scope)
        
        # Add timestamp
        parts.append(timestamp)
        
        return f"{'_'.join(parts)}.{file_format}"
    
    def _standardize_results_format(self, results: Dict[str, Any]) -> Dict[str, Any]:
        """Standardize results format for dashboard compatibility"""
        standardized = {}
        
        # Get service name from class
        service_name = getattr(self.service, '__class__', type(self.service)).__name__.lower()
        if 'auditservice' in service_name:
            service_name = service_name.replace('auditservice', '')
        
        for account_id, items in results.items():
            if isinstance(items, list):
                # Create dashboard-compatible structure: account -> service -> command -> {count, items}
                standardized[account_id] = {
                    service_name: {
                        'audit': {
                            'count': len(items),
                            'items': []
                        }
                    }
                }
                
                # Ensure all items have required fields for dashboard
                for item in items:
                    if isinstance(item, dict):
                        # Ensure Service field exists
                        if 'Service' not in item:
                            item['Service'] = service_name.upper()
                        
                        # Ensure AccountId field exists
                        if 'AccountId' not in item:
                            item['AccountId'] = account_id
                        
                        standardized[account_id][service_name]['audit']['items'].append(item)
            else:
                standardized[account_id] = items
        
        return standardized
        
    async def execute(self, method_name: str, output_format: str = 'console', save_to: Optional[str] = None, *args, **kwargs) -> Dict[str, Any]:
        # Validate save location upfront if specified
        if save_to and output_format in ['json', 'csv']:
            print(f"\n🔍 Validating save location: {save_to}")
            if not await self.storage_manager.validate_save_location(save_to):
                print("\n🛑 Save location validation failed. Aborting scan.")
                return {}
            print("✅ Save location validated successfully")
        
        # Display command description before starting
        self._display_command_description(method_name)
        
        spinner = SpinnerProgress(f"Running {method_name}")
        spinner.start()
        
        try:
            if self.organization:
                results = await self._execute_organization(method_name, *args, **kwargs)
            else:
                results = await self._execute_single_account(method_name, *args, **kwargs)
            
            # Display results based on output format
            await self._display_results(results, method_name, output_format, save_to)
            return results
        except ValueError as e:
            spinner.stop()
            print(f"\n{str(e)}")
            print("\n💡 Try running without --organization flag for single account scan.")
            return {}
        finally:
            spinner.stop()
    
    def _display_command_description(self, method_name: str):
        """Display what the command will do before execution"""
        descriptions = {
            'audit': '🔍 Running comprehensive audit (cost + security checks)',
            'cost_audit': '💰 Running cost optimization audit',
            'security_audit': '🔒 Running security audit',
            'check_empty_buckets': '🪣 Checking for empty S3 buckets',
            'check_unused_functions': '⚡ Checking for unused Lambda functions',
            'check_idle_instances': '💤 Checking for idle RDS instances',
            'check_oversized_instances': '📏 Checking for oversized instances',
            'check_ssh_open': '🚪 Checking for SSH ports open to internet',
            'check_public_read_access': '🌐 Checking for public read access',
            'check_unattached_eips': '🔗 Checking for unattached Elastic IPs',
            'check_unused_security_groups': '🛡️ Checking for unused security groups',
            'check_root_access_keys': '🔑 Checking for root account access keys',
            'check_old_access_keys': '⏰ Checking for old access keys',
            'check_orphan_volumes': '💾 Checking for orphaned EBS volumes',
            'check_unused_alarms': '⏰ Checking for unused CloudWatch alarms',
            'check_lbs_with_no_healthy_targets': '⚖️ Checking for load balancers with no targets'
        }
        
        description = descriptions.get(method_name, f'🔍 Running {method_name.replace("_", " ")}')
        scope = '🏢 Organization-wide' if self.organization else '📊 Single account'
        if isinstance(self.regions, list):
            regions_str = ', '.join(self.regions)
        else:
            regions_str = str(self.regions)
        region_info = f'📍 Regions: {regions_str}'
        
        print(f"\n{description}")
        print(f"{scope} | {region_info} | 👥 Workers: {self.max_workers}")
        print("─" * 60)
    
    async def _display_results(self, results: Dict[str, Any], method_name: str, output_format: str = 'console', save_to: Optional[str] = None):
        """Display results based on output format"""
        total_issues = 0
        
        for account_id, items in results.items():
            if isinstance(items, list):
                total_issues += len(items)
        
        # Always show console output with resource details
        for account_id, items in results.items():
            if isinstance(items, list) and items:
                print(f"\n📊 Account: {account_id}")
                print(f"  🔍 {method_name}: {len(items)} issues")
                
                # Display detailed items
                for item in items[:5]:  # Show first 5 items
                    if isinstance(item, dict):
                        resource_name = (item.get('ResourceName') or 
                                       item.get('Name') or
                                       item.get('BucketName') or 
                                       item.get('InstanceId') or 
                                       item.get('DBInstanceIdentifier') or 
                                       item.get('FunctionName') or 
                                       item.get('UserName') or 
                                       item.get('RoleName') or 
                                       item.get('VolumeId') or
                                       item.get('ResourceId') or
                                       'Unknown')
                        issue = item.get('Issue', 'Unknown issue')
                        severity = item.get('severity', item.get('Severity', 'Unknown'))
                        print(f"    • {resource_name}: {issue} [{severity}]")
                
                if len(items) > 5:
                    print(f"    ... and {len(items) - 5} more issues")
            elif isinstance(items, list) and not items:
                print(f"\n📊 Account: {account_id}")
                print(f"  ✅ {method_name}: No issues found")
        
        print(f"\n🎯 Total issues found: {total_issues}")
        
        # Handle output format
        if output_format == 'json':
            # Create standardized output format for dashboard compatibility
            json_output = {
                "scan_timestamp": datetime.now().isoformat(),
                "method": method_name,
                "total_issues": total_issues,
                "results": self._standardize_results_format(results)
            }
            
            filename = self._generate_filename(method_name, results, 'json')
            content = json.dumps(json_output, indent=2, default=str)
            
            if save_to:
                saved_location = await self.storage_manager.save_file(content, filename, save_to, 'json')
                print(f"\n📄 JSON report saved: {saved_location}")
            else:
                with open(filename, 'w') as f:
                    f.write(content)
                print(f"\n📄 JSON report saved: {filename}")
        
        elif output_format == 'csv':
            import csv
            from io import StringIO
            
            filename = self._generate_filename(method_name, results, 'csv')
            
            # Generate CSV content
            csv_content = ""
            if total_issues > 0:
                # Collect all possible fieldnames from all items
                all_fieldnames = set()
                all_items = []
                
                for items in results.values():
                    if isinstance(items, list):
                        for item in items:
                            if isinstance(item, dict):
                                all_fieldnames.update(item.keys())
                                all_items.append(item)
                
                if all_items:
                    output = StringIO()
                    fieldnames = sorted(all_fieldnames)
                    writer = csv.DictWriter(output, fieldnames=fieldnames, extrasaction='ignore')
                    writer.writeheader()
                    writer.writerows(all_items)
                    csv_content = output.getvalue()
                    output.close()
            
            if save_to:
                saved_location = await self.storage_manager.save_file(csv_content, filename, save_to, 'csv')
                print(f"\n📊 CSV report saved: {saved_location}")
            else:
                with open(filename, 'w', newline='') as f:
                    f.write(csv_content)
                print(f"\n📊 CSV report saved: {filename}")
    
    async def _execute_single_account(self, method_name: str, *args, **kwargs) -> Dict[str, Any]:
        session = boto3.Session()
        account_id = session.client('sts').get_caller_identity()['Account']
        
        all_results = []
        workers_per_region = max(1, self.max_workers // len(self.regions))
        
        for region in self.regions:
            loop = asyncio.get_event_loop()
            with ThreadPoolExecutor(max_workers=1) as executor:
                method = getattr(self.service, method_name)
                try:
                    result = await loop.run_in_executor(
                        executor, 
                        lambda r=region: method(session, r, max_workers=workers_per_region, *args, **kwargs)
                    )
                except TypeError:
                    result = await loop.run_in_executor(
                        executor, 
                        lambda r=region: method(session, r, *args, **kwargs)
                    )
                all_results.extend(result)
        
        return {account_id: all_results}
    
    async def _execute_organization(self, method_name: str, *args, **kwargs) -> Dict[str, Any]:
        accounts = await self._get_organization_accounts()
        print(f"\n🏢 Found {len(accounts)} accounts in organization")
        
        progress = ProgressBar(len(accounts), f"Scanning {method_name} across accounts")
        
        # Process accounts in batches for controlled concurrency
        semaphore = asyncio.Semaphore(self.max_workers)
        
        async def execute_with_semaphore(account_id):
            async with semaphore:
                result = await self._execute_for_account(account_id, method_name, *args, **kwargs)
                progress.update()
                return result
        
        tasks = [execute_with_semaphore(account_id) for account_id in accounts]
        results = await asyncio.gather(*tasks, return_exceptions=True)
        
        return dict(zip(accounts, results))
    
    async def _get_organization_accounts(self) -> List[str]:
        session = boto3.Session()
        
        # If org admin account is specified, assume role there first
        if self.org_admin_account_id:
            sts_client = session.client('sts')
            loop = asyncio.get_event_loop()
            
            with ThreadPoolExecutor() as executor:
                assumed_role = await loop.run_in_executor(
                    executor,
                    lambda: sts_client.assume_role(
                        RoleArn=f'arn:aws:iam::{self.org_admin_account_id}:role/{self.cross_account_role}',
                        RoleSessionName='kosty-org-admin'
                    )
                )
            
            session = boto3.Session(
                aws_access_key_id=assumed_role['Credentials']['AccessKeyId'],
                aws_secret_access_key=assumed_role['Credentials']['SecretAccessKey'],
                aws_session_token=assumed_role['Credentials']['SessionToken']
            )
        
        org_client = session.client('organizations')
        
        loop = asyncio.get_event_loop()
        with ThreadPoolExecutor() as executor:
            def get_all_accounts():
                try:
                    accounts = []
                    paginator = org_client.get_paginator('list_accounts')
                    for page in paginator.paginate():
                        accounts.extend(page['Accounts'])
                    return accounts
                except org_client.exceptions.AWSOrganizationsNotInUseException:
                    raise ValueError("❌ Account is not part of an AWS Organization. Use single account mode instead.")
                except Exception as e:
                    raise ValueError(f"❌ Failed to access organization: {str(e)}")
            
            all_accounts = await loop.run_in_executor(executor, get_all_accounts)
        
        return [account['Id'] for account in all_accounts if account['Status'] == 'ACTIVE']
    
    async def _execute_for_account(self, account_id: str, method_name: str, *args, **kwargs):
        try:
            session = boto3.Session()
            sts_client = session.client('sts')
            
            loop = asyncio.get_event_loop()
            with ThreadPoolExecutor() as executor:
                assumed_role = await loop.run_in_executor(
                    executor,
                    lambda: sts_client.assume_role(
                        RoleArn=f'arn:aws:iam::{account_id}:role/{self.cross_account_role}',
                        RoleSessionName=f'kosty-{account_id}'
                    )
                )
            
            assumed_session = boto3.Session(
                aws_access_key_id=assumed_role['Credentials']['AccessKeyId'],
                aws_secret_access_key=assumed_role['Credentials']['SecretAccessKey'],
                aws_session_token=assumed_role['Credentials']['SessionToken']
            )
            
            all_results = []
            workers_per_region = max(1, self.max_workers // len(self.regions))
            
            for region in self.regions:
                with ThreadPoolExecutor(max_workers=1) as executor:
                    method = getattr(self.service, method_name)
                    try:
                        result = await loop.run_in_executor(
                            executor,
                            lambda r=region: method(assumed_session, r, max_workers=workers_per_region, *args, **kwargs)
                        )
                    except TypeError:
                        result = await loop.run_in_executor(
                            executor,
                            lambda r=region: method(assumed_session, r, *args, **kwargs)
                        )
                    all_results.extend(result)
            
            return all_results
            
        except Exception as e:
            return f"Error: {str(e)}"


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def dynamodb(ctx):
    """DynamoDB operations"""
    pass

@dynamodb.command('audit')
@click.option('--days', default=7, help='Days threshold for idle tables')
@common_options
@click.pass_context
def dynamodb_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete DynamoDB audit"""
    from ..services.dynamodb_audit import DynamoDBAuditService
    execute_service_command(ctx, DynamoDBAuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@dynamodb.command('cost-audit')
@click.option('--days', default=7, help='Days threshold for idle tables')
@common_options
@click.pass_context
def dynamodb_cost_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run DynamoDB cost optimization audit only"""
    from ..services.dynamodb_audit import DynamoDBAuditService
    execute_service_command(ctx, DynamoDBAuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@dynamodb.command('security-audit')
@common_options
@click.pass_context
def dynamodb_security_audit(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run DynamoDB security audit only"""
    from ..services.dynamodb_audit import DynamoDBAuditService
    execute_service_command(ctx, DynamoDBAuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to)

@dynamodb.command('find-idle-tables')
@click.option('--days', default=7, help='Days threshold for table activity')
@common_options
@click.pass_context
def dynamodb_find_idle(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find idle DynamoDB tables (alternative method)"""
    from ..services.dynamodb_audit import DynamoDBAuditService
    execute_service_command(ctx, DynamoDBAuditService, 'find_idle_tables', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)


import json
import csv
from datetime import datetime
from typing import Dict, List, Any
from pathlib import Path

class CostOptimizationReporter:
    def __init__(self):
        self.results = {}
        self.scan_timestamp = datetime.now()
        self.organization = False
        self.org_admin_account_id = None
    
    def set_scan_context(self, organization: bool = False, org_admin_account_id: str = None):
        """Set scan context for proper filename generation"""
        self.organization = organization
        self.org_admin_account_id = org_admin_account_id
        
    def add_results(self, service: str, command: str, data: List[Dict[str, Any]], account_id: str = "current"):
        """Add scan results for a service"""
        if account_id not in self.results:
            self.results[account_id] = {}
        
        if service not in self.results[account_id]:
            self.results[account_id][service] = {}
            
        self.results[account_id][service][command] = {
            'count': len(data),
            'items': data,
            'potential_savings': self._calculate_savings(service, command, data)
        }
        
        # No cost calculation
    
    def _calculate_savings(self, service: str, command: str, data: List[Dict[str, Any]]) -> int:
        """No cost calculation - just count issues"""
        return len(data)
    

    
    def generate_summary_report(self) -> str:
        """Generate a summary report"""
        report = []
        report.append("\n" + "=" * 80)
        report.append("🏆 KOSTY - AWS COST OPTIMIZATION REPORT")
        report.append("=" * 80)
        report.append(f"📅 Scan Date: {self.scan_timestamp.strftime('%Y-%m-%d %H:%M:%S')}")
        report.append(f"📊 Total Issues Found: {sum(sum(cmd['count'] for cmd in svc.values()) for acc in self.results.values() for svc in acc.values())}")
        report.append("")
        
        # Summary by account
        for account_id, account_data in self.results.items():
            report.append(f"📊 Account: {account_id}")
            report.append("-" * 50)
            
            account_issues = 0
            total_issues = 0
            
            for service, service_data in account_data.items():
                for command, command_data in service_data.items():
                    count = command_data['count']
                    savings = command_data['potential_savings']
                    
                    if count > 0:
                        report.append(f"  🔍 {service.upper()} {command}: {count} issues")
                        account_issues += count
                        total_issues += count
            
            report.append(f"  💡 Account Total: {account_issues} issues found")
            report.append("")
        
        # Top issues by count
        report.append("🎯 TOP ISSUES BY COUNT")
        report.append("-" * 30)
        
        all_issues = []
        for account_id, account_data in self.results.items():
            for service, service_data in account_data.items():
                for command, command_data in service_data.items():
                    if command_data['count'] > 0:
                        all_issues.append({
                            'service': service,
                            'command': command,
                            'count': command_data['count']
                        })
        
        # Sort by issue count
        all_issues.sort(key=lambda x: x['count'], reverse=True)
        
        for i, issue in enumerate(all_issues[:10], 1):
            report.append(f"  {i:2d}. {issue['service'].upper()} {issue['command']}: {issue['count']} issues")
        
        report.append("")
        report.append("=" * 80)
        
        return "\n".join(report)
    
    async def save_json_report(self, filename: str = None, organization: bool = False, org_admin_account_id: str = None, save_to: str = None):
        """Save detailed JSON report"""
        from .storage import StorageManager
        
        if not filename:
            timestamp = self.scan_timestamp.strftime('%Y%m%d_%H%M%S')
            
            # Determine scope for filename
            if organization:
                if org_admin_account_id:
                    scope = f"org_{org_admin_account_id}"
                else:
                    scope = "org"
            else:
                # Single account - get first account ID
                account_id = list(self.results.keys())[0] if self.results else "unknown"
                scope = account_id
            
            filename = f"kosty_full_{scope}_{timestamp}.json"
        
        # Standardize results format for dashboard compatibility
        standardized_results = {}
        for account_id, account_data in self.results.items():
            standardized_results[account_id] = []
            for service, service_data in account_data.items():
                for command, command_data in service_data.items():
                    for item in command_data['items']:
                        if isinstance(item, dict):
                            # Ensure required fields for dashboard
                            if 'Service' not in item:
                                item['Service'] = service.upper()
                            if 'AccountId' not in item:
                                item['AccountId'] = account_id
                            standardized_results[account_id].append(item)
        
        report_data = {
            'scan_timestamp': self.scan_timestamp.isoformat(),
            'total_issues': sum(sum(cmd['count'] for cmd in svc.values()) for acc in self.results.values() for svc in acc.values()),
            'results': standardized_results,
            'summary': {
                'total_accounts': len(self.results),
                'total_issues': sum(
                    sum(cmd['count'] for cmd in svc.values())
                    for acc in self.results.values()
                    for svc in acc.values()
                )
            }
        }
        
        content = json.dumps(report_data, indent=2, default=str)
        
        if save_to:
            storage_manager = StorageManager()
            saved_location = await storage_manager.save_file(content, filename, save_to, 'json')
            return saved_location
        else:
            with open(filename, 'w') as f:
                f.write(content)
            return filename
    
    async def save_csv_report(self, filename: str = None, organization: bool = False, org_admin_account_id: str = None, save_to: str = None):
        """Save CSV report for spreadsheet analysis"""
        from .storage import StorageManager
        from io import StringIO
        
        if not filename:
            timestamp = self.scan_timestamp.strftime('%Y%m%d_%H%M%S')
            
            # Determine scope for filename
            if organization:
                if org_admin_account_id:
                    scope = f"org_{org_admin_account_id}"
                else:
                    scope = "org"
            else:
                # Single account - get first account ID
                account_id = list(self.results.keys())[0] if self.results else "unknown"
                scope = account_id
            
            filename = f"kosty_full_{scope}_{timestamp}.csv"
        
        # Generate CSV content
        output = StringIO()
        writer = csv.writer(output)
        writer.writerow(['Account', 'Service', 'Command', 'Resource Count', 'Cost Estimate', 'Details'])
        
        for account_id, account_data in self.results.items():
            for service, service_data in account_data.items():
                for command, command_data in service_data.items():
                    if command_data['count'] > 0:
                        details = '; '.join([
                            f"{item.get('InstanceId', item.get('VolumeId', item.get('FunctionName', item.get('ClusterName', 'Resource'))))}"
                            for item in command_data['items'][:5]  # First 5 items
                        ])
                        
                        writer.writerow([
                            account_id,
                            service,
                            command,
                            command_data['count'],
                            'N/A',  # No cost calculation
                            details
                        ])
        
        csv_content = output.getvalue()
        output.close()
        
        if save_to:
            storage_manager = StorageManager()
            saved_location = await storage_manager.save_file(csv_content, filename, save_to, 'csv')
            return saved_location
        else:
            with open(filename, 'w', newline='') as f:
                f.write(csv_content)
            return filename


import boto3
from typing import List, Dict, Any
from datetime import datetime, timedelta, timezone

class SnapshotsAuditService:
    service_name = "EBS Snapshots"
    
    cost_checks = [
        "check_old_snapshots"
    ]
    
    security_checks = []
    
    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run EBS Snapshots cost optimization audit"""
        results = []
        for check in self.cost_checks:
            results.extend(getattr(self, check)(session, region, **kwargs))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run EBS Snapshots security audit"""
        results = []
        for check in self.security_checks:
            results.extend(getattr(self, check)(session, region, **kwargs))
        return results
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run complete EBS Snapshots audit"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results
    
    def check_old_snapshots(self, session: boto3.Session, region: str, days: int = 30, **kwargs) -> List[Dict[str, Any]]:
        """Find EBS snapshots older than retention policy"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            response = ec2.describe_snapshots(OwnerIds=['self'])
            cutoff_date = datetime.now(timezone.utc) - timedelta(days=days)
            
            for snapshot in response['Snapshots']:
                if snapshot['StartTime'] < cutoff_date:
                    results.append({
                        'AccountId': session.client('sts').get_caller_identity()['Account'],
                        'Region': region,
                        'Service': self.service_name,
                        'ResourceId': snapshot['SnapshotId'],
                        'ResourceName': snapshot['SnapshotId'],
                        'Issue': 'Old EBS snapshot',
                        'type': 'cost',
                        'Risk': 'MEDIUM',
                        'severity': 'medium',
                        'Description': f"EBS snapshot {snapshot['SnapshotId']} is older than {days} days",
                        'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:snapshot/{snapshot['SnapshotId']}",
                        'Details': {
                            'SnapshotId': snapshot['SnapshotId'],
                            'VolumeId': snapshot.get('VolumeId', 'N/A'),
                            'StartTime': snapshot['StartTime'].isoformat(),
                            'VolumeSize': snapshot['VolumeSize'],
                            'State': snapshot['State'],
                            'Description': snapshot.get('Description', 'N/A')
                        }
                    })
        except Exception as e:
            print(f"Error checking old snapshots in {region}: {e}")
        
        return results

class SnapshotsService:
    # Legacy method for backward compatibility
    def find_old_snapshots(self, session: boto3.Session, region: str, days: int = 30) -> List[Dict[str, Any]]:
        return SnapshotsAuditService().check_old_snapshots(session, region, days=days)


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='kosty',
    version='1.3.8',
    author='Yassir Kachri',
    author_email='yassir@kosty.cloud',
    description='AWS Cost Optimization & Security Audit CLI Tool - Identify cost waste, security vulnerabilities, and compliance issues across 16 core AWS services',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/kosty-cloud/kosty',
    project_urls={
        'Bug Reports': 'https://github.com/kosty-cloud/kosty/issues',
        'Source': 'https://github.com/kosty-cloud/kosty',
        'Documentation': 'https://github.com/kosty-cloud/kosty/blob/main/docs/DOCUMENTATION.md',
    },
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
    ],
    keywords='aws cost optimization security audit cloud infrastructure compliance finops devops',
    install_requires=[
        'click>=8.0.0',
        'boto3>=1.26.0',
    ],
    entry_points={
        'console_scripts': [
            'kosty=kosty.cli:cli',
        ],
    },
    python_requires='>=3.7',
    include_package_data=True,
    zip_safe=False,
)


import boto3
from typing import List, Dict, Any

class SGAuditService:
    def __init__(self):
        self.cost_checks = ['find_unused_security_groups']
        self.security_checks = ['find_ssh_rdp_open', 'find_database_ports_open', 'find_all_ports_open']
        self.other_checks = ['find_complex_security_groups']

    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all cost-related audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            results.extend(method(session, region))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all security-related audits"""
        results = []
        for check in self.security_checks:
            method = getattr(self, check)
            results.extend(method(session, region))
        return results

    def other_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all other-related audits"""
        results = []
        for check in self.other_checks:
            method = getattr(self, check)
            if check == 'find_complex_security_groups':
                results.extend(method(session, region, **kwargs))
            else:
                results.extend(method(session, region))
        return results
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all audits (cost + security + other)"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        results.extend(self.other_audit(session, region, **kwargs))
        return results

    def find_ssh_rdp_open(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find security groups with SSH/RDP open to 0.0.0.0/0"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            response = ec2.describe_security_groups()
            
            for sg in response['SecurityGroups']:
                for rule in sg.get('IpPermissions', []):
                    for ip_range in rule.get('IpRanges', []):
                        if ip_range.get('CidrIp') == '0.0.0.0/0':
                            from_port = rule.get('FromPort', 0)
                            to_port = rule.get('ToPort', 65535)
                            
                            if (22 >= from_port and 22 <= to_port) or (3389 >= from_port and 3389 <= to_port):
                                results.append({
                                    'AccountId': session.client('sts').get_caller_identity()['Account'],
                                    'ResourceName': sg['GroupName'],
                                    'Region': region,
                                    'Service': 'SG',
                                    'ResourceId': sg['GroupId'],
                                    'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:security-group/{sg['GroupId']}",
                                    'Issue': 'Port 22/3389 open to 0.0.0.0/0',
                                    'type': 'security',
                                    'Risk': 'Brute force attacks',
                                    'severity': 'critical',
                                    'Details': {
                                        'GroupName': sg['GroupName'],
                                        'FromPort': from_port,
                                        'ToPort': to_port,
                                        'Protocol': rule.get('IpProtocol')
                                    }
                                })
                                break
        except Exception as e:
            pass
        
        return results

    def find_database_ports_open(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find security groups with database ports open to 0.0.0.0/0"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            response = ec2.describe_security_groups()
            db_ports = [3306, 5432, 1433, 27017]
            
            for sg in response['SecurityGroups']:
                for rule in sg.get('IpPermissions', []):
                    for ip_range in rule.get('IpRanges', []):
                        if ip_range.get('CidrIp') == '0.0.0.0/0':
                            from_port = rule.get('FromPort', 0)
                            to_port = rule.get('ToPort', 65535)
                            
                            if any(port >= from_port and port <= to_port for port in db_ports):
                                results.append({
                                    'AccountId': session.client('sts').get_caller_identity()['Account'],
                                    'ResourceName': sg['GroupName'],
                                    'Region': region,
                                    'Service': 'SG',
                                    'ResourceId': sg['GroupId'],
                                    'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:security-group/{sg['GroupId']}",
                                    'Issue': 'Database ports open to 0.0.0.0/0',
                                    'type': 'security',
                                    'Risk': 'Unauthorized DB access',
                                    'severity': 'critical',
                                    'Details': {
                                        'GroupName': sg['GroupName'],
                                        'FromPort': from_port,
                                        'ToPort': to_port,
                                        'Protocol': rule.get('IpProtocol')
                                    }
                                })
                                break
        except Exception as e:
            pass
        
        return results

    def find_all_ports_open(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find security groups with all ports open to 0.0.0.0/0"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            response = ec2.describe_security_groups()
            
            for sg in response['SecurityGroups']:
                for rule in sg.get('IpPermissions', []):
                    for ip_range in rule.get('IpRanges', []):
                        if ip_range.get('CidrIp') == '0.0.0.0/0':
                            from_port = rule.get('FromPort', 0)
                            to_port = rule.get('ToPort', 65535)
                            
                            if from_port == 0 and to_port == 65535:
                                results.append({
                                    'AccountId': session.client('sts').get_caller_identity()['Account'],
                                    'ResourceName': sg['GroupName'],
                                    'Region': region,
                                    'Service': 'SG',
                                    'ResourceId': sg['GroupId'],
                                    'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:security-group/{sg['GroupId']}",
                                    'Issue': 'All ports open (0-65535) to 0.0.0.0/0',
                                    'type': 'security',
                                    'Risk': 'Complete exposure',
                                    'severity': 'critical',
                                    'Details': {
                                        'GroupName': sg['GroupName'],
                                        'Protocol': rule.get('IpProtocol')
                                    }
                                })
                                break
        except Exception as e:
            pass
        
        return results

    def find_unused_security_groups(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find unused security groups"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            # Get all security groups
            sgs_response = ec2.describe_security_groups()
            all_sgs = {sg['GroupId']: sg for sg in sgs_response['SecurityGroups']}
            
            # Get security groups in use by instances
            used_sgs = set()
            instances_response = ec2.describe_instances()
            for reservation in instances_response['Reservations']:
                for instance in reservation['Instances']:
                    for sg in instance.get('SecurityGroups', []):
                        used_sgs.add(sg['GroupId'])
            
            # Get security groups in use by load balancers
            try:
                elbv2 = session.client('elbv2', region_name=region)
                lbs_response = elbv2.describe_load_balancers()
                for lb in lbs_response['LoadBalancers']:
                    for sg_id in lb.get('SecurityGroups', []):
                        used_sgs.add(sg_id)
            except Exception:
                pass
            
            # Find unused security groups
            for sg_id, sg in all_sgs.items():
                if sg_id not in used_sgs and sg['GroupName'] != 'default':
                    results.append({
                        'AccountId': session.client('sts').get_caller_identity()['Account'],
                        'ResourceName': sg['GroupName'],
                        'Region': region,
                        'Service': 'SG',
                        'ResourceId': sg_id,
                        'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:security-group/{sg_id}",
                        'Issue': 'Unused security group (no attachments)',
                        'type': 'cost',
                        'Risk': 'Configuration drift',
                        'severity': 'low',
                        'Details': {
                            'GroupName': sg['GroupName'],
                            'Description': sg.get('Description', '')
                        }
                    })
        except Exception as e:
            pass
        
        return results

    def find_complex_security_groups(self, session: boto3.Session, region: str, rule_threshold: int = 50) -> List[Dict[str, Any]]:
        """Find security groups with >rule_threshold rules"""
        ec2 = session.client('ec2', region_name=region)
        results = []
        
        try:
            response = ec2.describe_security_groups()
            
            for sg in response['SecurityGroups']:
                rule_count = len(sg.get('IpPermissions', [])) + len(sg.get('IpPermissionsEgress', []))
                
                if rule_count > rule_threshold:
                    results.append({
                        'AccountId': session.client('sts').get_caller_identity()['Account'],
                        'ResourceName': sg['GroupName'],
                        'Region': region,
                        'Service': 'SG',
                        'ResourceId': sg['GroupId'],
                        'ARN': f"arn:aws:ec2:{region}:{session.client('sts').get_caller_identity()['Account']}:security-group/{sg['GroupId']}",
                        'Issue': f'Security group with >{rule_threshold} rules',
                        'type': 'other',
                        'Risk': 'Complex troubleshooting',
                        'severity': 'low',
                        'Details': {
                            'GroupName': sg['GroupName'],
                            'RuleCount': rule_count,
                            'Threshold': rule_threshold
                        }
                    })
        except Exception as e:
            pass
        
        return results

    # Individual check aliases
    def check_unused_groups(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_unused_security_groups(session, region)
    
    def check_ssh_rdp_open(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_ssh_rdp_open(session, region)
    
    def check_database_ports_open(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_database_ports_open(session, region)
    
    def check_all_ports_open(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_all_ports_open(session, region)
    
    def check_complex_security_groups(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_complex_security_groups(session, region, **kwargs)


import click
from .utils import common_options, execute_service_command

@click.group()
@click.pass_context
def ec2(ctx):
    """EC2 operations"""
    pass

@ec2.command('audit')
@click.option('--days', default=7, help='Days threshold for stopped/idle instances')
@click.option('--cpu-threshold', default=20, help='CPU utilization threshold')
@common_options
@click.pass_context
def ec2_audit(ctx, days, cpu_threshold, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run complete EC2 audit (cost + security)"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days, cpu_threshold=cpu_threshold)

@ec2.command('cost-audit')
@click.option('--days', default=7, help='Days threshold for stopped/idle instances')
@click.option('--cpu-threshold', default=20, help='CPU utilization threshold')
@common_options
@click.pass_context
def ec2_cost_audit(ctx, days, cpu_threshold, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run EC2 cost optimization audit only"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'cost_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days, cpu_threshold=cpu_threshold)

@ec2.command('security-audit')
@click.option('--days', default=180, help='Days threshold for AMI age')
@common_options
@click.pass_context
def ec2_security_audit(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Run EC2 security audit only"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'security_audit', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

# Individual EC2 checks
@ec2.command('check-oversized-instances')
@click.option('--cpu-threshold', default=20, help='CPU utilization threshold')
@click.option('--days', default=14, help='Days to analyze')
@common_options
@click.pass_context
def ec2_check_oversized(ctx, cpu_threshold, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find oversized EC2 instances"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_oversized_instances', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, cpu_threshold=cpu_threshold, days=days)

@ec2.command('check-stopped-instances')
@click.option('--days', default=7, help='Days threshold for stopped instances')
@common_options
@click.pass_context
def ec2_check_stopped(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find stopped EC2 instances"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_stopped_instances', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@ec2.command('check-idle-instances')
@click.option('--cpu-threshold', default=5, help='CPU utilization threshold')
@click.option('--days', default=7, help='Days to analyze')
@common_options
@click.pass_context
def ec2_check_idle(ctx, cpu_threshold, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find idle EC2 instances"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_idle_instances', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, cpu_threshold=cpu_threshold, days=days)

@ec2.command('check-previous-generation')
@common_options
@click.pass_context
def ec2_check_previous_gen(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find previous generation instances (t2/m4/c4)"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_previous_generation', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

# Security checks
@ec2.command('check-ssh-open')
@common_options
@click.pass_context
def ec2_check_ssh(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances with SSH open to 0.0.0.0/0"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_ssh_open', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@ec2.command('check-rdp-open')
@common_options
@click.pass_context
def ec2_check_rdp(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances with RDP open to 0.0.0.0/0"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_rdp_open', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@ec2.command('check-database-ports-open')
@common_options
@click.pass_context
def ec2_check_db_ports(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances with database ports open to 0.0.0.0/0"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_database_ports_open', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@ec2.command('check-public-non-web')
@common_options
@click.pass_context
def ec2_check_public_non_web(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find public IP on non-web instances"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_public_non_web', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@ec2.command('check-old-ami')
@click.option('--days', default=180, help='Days threshold for AMI age')
@common_options
@click.pass_context
def ec2_check_old_ami(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances using AMI older than X days"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_old_ami', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)

@ec2.command('check-imdsv1')
@common_options
@click.pass_context
def ec2_check_imdsv1(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances using IMDSv1"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_imdsv1', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@ec2.command('check-unencrypted-ebs')
@common_options
@click.pass_context
def ec2_check_unencrypted_ebs(ctx, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances with unencrypted EBS volumes"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_unencrypted_ebs', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id)

@ec2.command('check-no-recent-backup')
@click.option('--days', default=30, help='Days threshold for recent AMI backup')
@common_options
@click.pass_context
def ec2_check_no_backup(ctx, days, organization, region, max_workers, regions, output, save_to, cross_account_role, org_admin_account_id):
    """Find instances with no recent AMI backup"""
    from ..services.ec2_audit import EC2AuditService
    execute_service_command(ctx, EC2AuditService, 'check_no_recent_backup', output, organization, region, max_workers, regions, cross_account_role, org_admin_account_id, save_to, days=days)


import boto3
from typing import List, Dict, Any
import json

class S3AuditService:
    def __init__(self):
        self.cost_checks = ['find_empty', 'find_incomplete_uploads', 'find_lifecycle_candidates']
        self.security_checks = ['find_public_read', 'find_public_write', 'find_no_encryption', 
                               'find_no_versioning', 'find_no_logging', 'find_wildcard_policy', 
                               'find_public_snapshots', 'find_no_mfa_delete']
    
    # Cost Audit Methods
    def find_empty(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find empty S3 buckets"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        empty_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    # Get bucket location
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    # Check if bucket has objects
                    objects = s3.list_objects_v2(Bucket=bucket_name, MaxKeys=1)
                    if 'Contents' not in objects:
                        empty_buckets.append({
                            'AccountId': account_id,
                            'ResourceName': bucket_name,
                            'BucketName': bucket_name,
                            'ARN': f'arn:aws:s3:::{bucket_name}',
                            'CreationDate': bucket['CreationDate'].isoformat(),
                            'Region': bucket_region,
                            'Issue': 'Empty bucket with no objects',
                            'type': 'cost',
                            'Risk': 'Waste $0.50-5/mo per bucket',
                            'severity': 'low',
                            'Service': 'S3'
                        })
                except Exception as e:
                    continue
        except Exception as e:
            print(f"Error listing buckets: {e}")
        
        return empty_buckets
    
    def find_incomplete_uploads(self, session: boto3.Session, region: str, days: int = 7) -> List[Dict[str, Any]]:
        """Find incomplete multipart uploads"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        incomplete_uploads = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    uploads = s3.list_multipart_uploads(Bucket=bucket_name)
                    if 'Uploads' in uploads and uploads['Uploads']:
                        incomplete_uploads.append({
                            'AccountId': account_id,
                            'ResourceName': bucket_name,
                            'BucketName': bucket_name,
                            'ARN': f'arn:aws:s3:::{bucket_name}',
                            'Region': bucket_region,
                            'IncompleteUploads': len(uploads['Uploads']),
                            'Issue': 'Incomplete multipart uploads',
                            'type': 'cost',
                            'Risk': 'Waste $10-100/mo per bucket',
                            'severity': 'medium',
                            'Service': 'S3'
                        })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking incomplete uploads: {e}")
        
        return incomplete_uploads
    
    def find_lifecycle_candidates(self, session: boto3.Session, region: str, days: int = 90) -> List[Dict[str, Any]]:
        """Find buckets needing lifecycle policies"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        lifecycle_candidates = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    # Check if lifecycle policy exists
                    try:
                        s3.get_bucket_lifecycle_configuration(Bucket=bucket_name)
                        has_lifecycle = True
                    except s3.exceptions.NoSuchLifecycleConfiguration:
                        has_lifecycle = False
                    
                    if not has_lifecycle:
                        # Check for old objects
                        objects = s3.list_objects_v2(Bucket=bucket_name, MaxKeys=10)
                        if 'Contents' in objects:
                            lifecycle_candidates.append({
                                'AccountId': account_id,
                                'ResourceName': bucket_name,
                                'BucketName': bucket_name,
                                'ARN': f'arn:aws:s3:::{bucket_name}',
                                'Region': bucket_region,
                                'ObjectCount': len(objects['Contents']),
                                'Issue': 'No lifecycle policy on old data (90+ days)',
                                'type': 'cost',
                                'Risk': 'Waste 50-70% storage costs',
                                'severity': 'high',
                                'Service': 'S3'
                            })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking lifecycle policies: {e}")
        
        return lifecycle_candidates
    
    # Security Audit Methods
    def find_public_read(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets with public read access"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        public_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    # Check bucket ACL
                    acl = s3.get_bucket_acl(Bucket=bucket_name)
                    for grant in acl['Grants']:
                        grantee = grant.get('Grantee', {})
                        if grantee.get('URI') == 'http://acs.amazonaws.com/groups/global/AllUsers':
                            if 'READ' in grant['Permission']:
                                public_buckets.append({
                                    'AccountId': account_id,
                                    'ResourceName': bucket_name,
                                    'BucketName': bucket_name,
                                    'ARN': f'arn:aws:s3:::{bucket_name}',
                                    'Region': bucket_region,
                                    'Issue': 'Public read access enabled',
                                    'type': 'security',
                                    'Risk': 'Data breach - complete bucket exposure',
                                    'severity': 'critical',
                                    'Service': 'S3',
                                    'Permission': grant['Permission']
                                })
                                break
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking public read access: {e}")
        
        return public_buckets
    
    def find_public_write(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets with public write access"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        public_write_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    acl = s3.get_bucket_acl(Bucket=bucket_name)
                    for grant in acl['Grants']:
                        grantee = grant.get('Grantee', {})
                        if grantee.get('URI') == 'http://acs.amazonaws.com/groups/global/AllUsers':
                            if 'WRITE' in grant['Permission']:
                                public_write_buckets.append({
                                    'AccountId': account_id,
                                    'ResourceName': bucket_name,
                                    'BucketName': bucket_name,
                                    'ARN': f'arn:aws:s3:::{bucket_name}',
                                    'Region': bucket_region,
                                    'Issue': 'Public write access enabled',
                                    'type': 'security',
                                    'Risk': 'Malware injection - crypto mining',
                                    'severity': 'critical',
                                    'Service': 'S3'
                                })
                                break
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking public write access: {e}")
        
        return public_write_buckets
    
    def find_no_encryption(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets without encryption"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        unencrypted_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    s3.get_bucket_encryption(Bucket=bucket_name)
                except s3.exceptions.NoSuchBucketEncryption:
                    unencrypted_buckets.append({
                        'AccountId': account_id,
                        'ResourceName': bucket_name,
                        'BucketName': bucket_name,
                        'ARN': f'arn:aws:s3:::{bucket_name}',
                        'Region': bucket_region,
                        'Issue': 'No encryption at rest',
                        'type': 'security',
                        'Risk': 'Data exposure if storage compromised',
                        'severity': 'critical',
                        'Service': 'S3'
                    })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking encryption: {e}")
        
        return unencrypted_buckets
    
    def find_no_versioning(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets without versioning"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        no_versioning_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    versioning = s3.get_bucket_versioning(Bucket=bucket_name)
                    if versioning.get('Status') != 'Enabled':
                        no_versioning_buckets.append({
                            'AccountId': account_id,
                            'ResourceName': bucket_name,
                            'BucketName': bucket_name,
                            'ARN': f'arn:aws:s3:::{bucket_name}',
                            'Region': bucket_region,
                            'Issue': 'Versioning disabled',
                            'type': 'security',
                            'Risk': 'No ransomware protection',
                            'severity': 'high',
                            'Service': 'S3'
                        })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking versioning: {e}")
        
        return no_versioning_buckets
    
    def find_no_logging(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets without access logging"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        no_logging_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    logging = s3.get_bucket_logging(Bucket=bucket_name)
                    if 'LoggingEnabled' not in logging:
                        no_logging_buckets.append({
                            'AccountId': account_id,
                            'ResourceName': bucket_name,
                            'BucketName': bucket_name,
                            'ARN': f'arn:aws:s3:::{bucket_name}',
                            'Region': bucket_region,
                            'Issue': 'No access logging',
                            'type': 'security',
                            'Risk': 'Cannot audit data access',
                            'severity': 'medium',
                            'Service': 'S3'
                        })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking logging: {e}")
        
        return no_logging_buckets
    
    def find_wildcard_policy(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets with wildcard policies"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        wildcard_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    policy = s3.get_bucket_policy(Bucket=bucket_name)
                    policy_doc = json.loads(policy['Policy'])
                    
                    for statement in policy_doc.get('Statement', []):
                        principal = statement.get('Principal')
                        if principal == '*' or (isinstance(principal, dict) and principal.get('AWS') == '*'):
                            wildcard_buckets.append({
                                'AccountId': account_id,
                                'ResourceName': bucket_name,
                                'BucketName': bucket_name,
                                'ARN': f'arn:aws:s3:::{bucket_name}',
                                'Region': bucket_region,
                                'Issue': 'Bucket policy allows wildcard principal (*)',
                                'type': 'security',
                                'Risk': 'Overly permissive access',
                                'severity': 'high',
                                'Service': 'S3'
                            })
                            break
                except Exception as e:
                    if 'NoSuchBucketPolicy' in str(e):
                        continue
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking bucket policies: {e}")
        
        return wildcard_buckets
    
    def find_public_snapshots(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find public snapshots (placeholder - S3 doesn't have snapshots)"""
        # This is more relevant for EBS, but keeping for consistency
        return []
    
    def find_no_mfa_delete(self, session: boto3.Session, region: str) -> List[Dict[str, Any]]:
        """Find buckets without MFA delete"""
        s3 = session.client('s3')
        sts = session.client('sts')
        account_id = sts.get_caller_identity()['Account']
        no_mfa_buckets = []
        
        try:
            buckets = s3.list_buckets()['Buckets']
            for bucket in buckets:
                bucket_name = bucket['Name']
                try:
                    location = s3.get_bucket_location(Bucket=bucket_name)
                    bucket_region = location['LocationConstraint'] or 'us-east-1'
                    
                    versioning = s3.get_bucket_versioning(Bucket=bucket_name)
                    if versioning.get('MfaDelete') != 'Enabled':
                        no_mfa_buckets.append({
                            'AccountId': account_id,
                            'ResourceName': bucket_name,
                            'BucketName': bucket_name,
                            'ARN': f'arn:aws:s3:::{bucket_name}',
                            'Region': bucket_region,
                            'Issue': 'No MFA delete enabled',
                            'type': 'security',
                            'Risk': 'Accidental/malicious deletion',
                            'severity': 'medium',
                            'Service': 'S3',
                            'MfaDeleteStatus': versioning.get('MfaDelete', 'Disabled')
                        })
                except Exception:
                    continue
        except Exception as e:
            print(f"Error checking MFA delete: {e}")
        
        return no_mfa_buckets
    
    # Audit Methods
    def cost_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all cost-related audits"""
        results = []
        for check in self.cost_checks:
            method = getattr(self, check)
            if check in ['find_incomplete_uploads', 'find_lifecycle_candidates']:
                results.extend(method(session, region, **kwargs))
            else:
                results.extend(method(session, region))
        return results
    
    def security_audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all security-related audits"""
        results = []
        for check in self.security_checks:
            method = getattr(self, check)
            results.extend(method(session, region))
        return results
    
    def audit(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        """Run all audits (cost + security)"""
        results = []
        results.extend(self.cost_audit(session, region, **kwargs))
        results.extend(self.security_audit(session, region, **kwargs))
        return results
    
    # Individual Check Method Aliases
    def check_empty_buckets(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_empty(session, region)
    
    def check_incomplete_uploads(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_incomplete_uploads(session, region, **kwargs)
    
    def check_lifecycle_policy(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_lifecycle_candidates(session, region, **kwargs)
    
    def check_public_read_access(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_public_read(session, region)
    
    def check_public_write_access(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_public_write(session, region)
    
    def check_encryption_at_rest(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_no_encryption(session, region)
    
    def check_versioning_disabled(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_no_versioning(session, region)
    
    def check_access_logging(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_no_logging(session, region)
    
    def check_bucket_policy_wildcards(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_wildcard_policy(session, region)
    
    def check_public_snapshots(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_public_snapshots(session, region)
    
    def check_mfa_delete(self, session: boto3.Session, region: str, **kwargs) -> List[Dict[str, Any]]:
        return self.find_no_mfa_delete(session, region)